"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LoadNodesAndCredentials = void 0;
const n8n_core_1 = require("n8n-core");
const config = require("../config");
const fs_1 = require("fs");
const glob = require("glob-promise");
const path = require("path");
const util_1 = require("util");
const fsAccessAsync = util_1.promisify(fs_1.access);
const fsReaddirAsync = util_1.promisify(fs_1.readdir);
const fsReadFileAsync = util_1.promisify(fs_1.readFile);
const fsStatAsync = util_1.promisify(fs_1.stat);
class LoadNodesAndCredentialsClass {
    constructor() {
        this.nodeTypes = {};
        this.credentialTypes = {};
        this.excludeNodes = undefined;
        this.nodeModulesPath = '';
    }
    async init() {
        const checkPaths = [
            path.join(__dirname, '..', '..', '..', 'n8n-workflow'),
            path.join(__dirname, '..', '..', 'node_modules', 'n8n-workflow'),
        ];
        for (const checkPath of checkPaths) {
            try {
                await fsAccessAsync(checkPath);
                this.nodeModulesPath = path.dirname(checkPath);
                break;
            }
            catch (error) {
                continue;
            }
        }
        if (this.nodeModulesPath === '') {
            throw new Error('Could not find "node_modules" folder!');
        }
        this.excludeNodes = config.get('nodes.exclude');
        const packages = await this.getN8nNodePackages();
        for (const packageName of packages) {
            await this.loadDataFromPackage(packageName);
        }
        const customDirectories = [];
        customDirectories.push(n8n_core_1.UserSettings.getUserN8nFolderCustomExtensionPath());
        if (process.env[n8n_core_1.CUSTOM_EXTENSION_ENV] !== undefined) {
            const customExtensionFolders = process.env[n8n_core_1.CUSTOM_EXTENSION_ENV].split(';');
            customDirectories.push.apply(customDirectories, customExtensionFolders);
        }
        for (const directory of customDirectories) {
            await this.loadDataFromDirectory('CUSTOM', directory);
        }
    }
    async getN8nNodePackages() {
        const getN8nNodePackagesRecursive = async (relativePath) => {
            const results = [];
            const nodeModulesPath = `${this.nodeModulesPath}/${relativePath}`;
            for (const file of await fsReaddirAsync(nodeModulesPath)) {
                const isN8nNodesPackage = file.indexOf('n8n-nodes-') === 0;
                const isNpmScopedPackage = file.indexOf('@') === 0;
                if (!isN8nNodesPackage && !isNpmScopedPackage) {
                    continue;
                }
                if (!(await fsStatAsync(nodeModulesPath)).isDirectory()) {
                    continue;
                }
                if (isN8nNodesPackage) {
                    results.push(`${relativePath}${file}`);
                }
                if (isNpmScopedPackage) {
                    results.push(...await getN8nNodePackagesRecursive(`${relativePath}${file}/`));
                }
            }
            return results;
        };
        return getN8nNodePackagesRecursive('');
    }
    async loadCredentialsFromFile(credentialName, filePath) {
        const tempModule = require(filePath);
        let tempCredential;
        try {
            tempCredential = new tempModule[credentialName]();
        }
        catch (e) {
            if (e instanceof TypeError) {
                throw new Error(`Class with name "${credentialName}" could not be found. Please check if the class is named correctly!`);
            }
            else {
                throw e;
            }
        }
        this.credentialTypes[tempCredential.name] = tempCredential;
    }
    async loadNodeFromFile(packageName, nodeName, filePath) {
        let tempNode;
        let fullNodeName;
        const tempModule = require(filePath);
        try {
            tempNode = new tempModule[nodeName]();
        }
        catch (error) {
            console.error(`Error loading node "${nodeName}" from: "${filePath}"`);
            throw error;
        }
        fullNodeName = packageName + '.' + tempNode.description.name;
        tempNode.description.name = fullNodeName;
        if (tempNode.description.icon !== undefined &&
            tempNode.description.icon.startsWith('file:')) {
            tempNode.description.icon = 'file:' + path.join(path.dirname(filePath), tempNode.description.icon.substr(5));
        }
        if (this.excludeNodes !== undefined && this.excludeNodes.includes(fullNodeName)) {
            return;
        }
        this.nodeTypes[fullNodeName] = {
            type: tempNode,
            sourcePath: filePath,
        };
    }
    async loadDataFromDirectory(setPackageName, directory) {
        const files = await glob(path.join(directory, '**/*\.@(node|credentials)\.js'));
        let fileName;
        let type;
        const loadPromises = [];
        for (const filePath of files) {
            [fileName, type] = path.parse(filePath).name.split('.');
            if (type === 'node') {
                loadPromises.push(this.loadNodeFromFile(setPackageName, fileName, filePath));
            }
            else if (type === 'credentials') {
                loadPromises.push(this.loadCredentialsFromFile(fileName, filePath));
            }
        }
        await Promise.all(loadPromises);
    }
    async loadDataFromPackage(packageName) {
        const packagePath = path.join(this.nodeModulesPath, packageName);
        const packageFileString = await fsReadFileAsync(path.join(packagePath, 'package.json'), 'utf8');
        const packageFile = JSON.parse(packageFileString);
        if (!packageFile.hasOwnProperty('n8n')) {
            return;
        }
        let tempPath, filePath;
        let fileName, type;
        if (packageFile.n8n.hasOwnProperty('nodes') && Array.isArray(packageFile.n8n.nodes)) {
            for (filePath of packageFile.n8n.nodes) {
                tempPath = path.join(packagePath, filePath);
                [fileName, type] = path.parse(filePath).name.split('.');
                await this.loadNodeFromFile(packageName, fileName, tempPath);
            }
        }
        if (packageFile.n8n.hasOwnProperty('credentials') && Array.isArray(packageFile.n8n.credentials)) {
            for (filePath of packageFile.n8n.credentials) {
                tempPath = path.join(packagePath, filePath);
                [fileName, type] = path.parse(filePath).name.split('.');
                this.loadCredentialsFromFile(fileName, tempPath);
            }
        }
    }
}
let packagesInformationInstance;
function LoadNodesAndCredentials() {
    if (packagesInformationInstance === undefined) {
        packagesInformationInstance = new LoadNodesAndCredentialsClass();
    }
    return packagesInformationInstance;
}
exports.LoadNodesAndCredentials = LoadNodesAndCredentials;
//# sourceMappingURL=LoadNodesAndCredentials.js.map