"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WorkflowRunner = void 0;
const _1 = require("./");
const n8n_core_1 = require("n8n-core");
const n8n_workflow_1 = require("n8n-workflow");
const config = require("../config");
const path_1 = require("path");
const child_process_1 = require("child_process");
class WorkflowRunner {
    constructor() {
        this.push = _1.Push.getInstance();
        this.activeExecutions = _1.ActiveExecutions.getInstance();
        this.credentialsOverwrites = _1.CredentialsOverwrites().getAll();
    }
    processHookMessage(workflowHooks, hookData) {
        workflowHooks.executeHookFunctions(hookData.hook, hookData.parameters);
    }
    processError(error, startedAt, executionMode, executionId) {
        const fullRunData = {
            data: {
                resultData: {
                    error,
                    runData: {},
                },
            },
            finished: false,
            mode: executionMode,
            startedAt,
            stoppedAt: new Date(),
        };
        this.activeExecutions.remove(executionId, fullRunData);
        _1.WorkflowExecuteAdditionalData.pushExecutionFinished(executionMode, fullRunData, executionId);
    }
    async run(data, loadStaticData) {
        const externalHooks = _1.ExternalHooks();
        await externalHooks.run('workflow.execute', [data.workflowData, data.executionMode]);
        const executionsProcess = config.get('executions.process');
        let executionId;
        if (executionsProcess === 'main') {
            executionId = await this.runMainProcess(data, loadStaticData);
        }
        else {
            executionId = await this.runSubprocess(data, loadStaticData);
        }
        if (externalHooks.exists('workflow.postExecute')) {
            this.activeExecutions.getPostExecutePromise(executionId)
                .then(async (executionData) => {
                await externalHooks.run('workflow.postExecute', [executionData, data.workflowData]);
            })
                .catch(error => {
                console.error('There was a problem running hook "workflow.postExecute"', error);
            });
        }
        return executionId;
    }
    async runMainProcess(data, loadStaticData) {
        if (loadStaticData === true && data.workflowData.id) {
            data.workflowData.staticData = await _1.WorkflowHelpers.getStaticDataById(data.workflowData.id);
        }
        const nodeTypes = _1.NodeTypes();
        const workflow = new n8n_workflow_1.Workflow({ id: data.workflowData.id, name: data.workflowData.name, nodes: data.workflowData.nodes, connections: data.workflowData.connections, active: data.workflowData.active, nodeTypes, staticData: data.workflowData.staticData });
        const additionalData = await _1.WorkflowExecuteAdditionalData.getBase(data.credentials);
        const executionId = this.activeExecutions.add(data, undefined);
        additionalData.hooks = _1.WorkflowExecuteAdditionalData.getWorkflowHooksMain(data, executionId);
        let workflowExecution;
        if (data.executionData !== undefined) {
            const workflowExecute = new n8n_core_1.WorkflowExecute(additionalData, data.executionMode, data.executionData);
            workflowExecution = workflowExecute.processRunExecutionData(workflow);
        }
        else if (data.runData === undefined || data.startNodes === undefined || data.startNodes.length === 0 || data.destinationNode === undefined) {
            const workflowExecute = new n8n_core_1.WorkflowExecute(additionalData, data.executionMode);
            workflowExecution = workflowExecute.run(workflow, undefined, data.destinationNode);
        }
        else {
            const workflowExecute = new n8n_core_1.WorkflowExecute(additionalData, data.executionMode);
            workflowExecution = workflowExecute.runPartialWorkflow(workflow, data.runData, data.startNodes, data.destinationNode);
        }
        this.activeExecutions.attachWorkflowExecution(executionId, workflowExecution);
        let executionTimeout;
        let workflowTimeout = config.get('executions.timeout') > 0 && config.get('executions.timeout');
        if (data.workflowData.settings && data.workflowData.settings.executionTimeout) {
            workflowTimeout = data.workflowData.settings.executionTimeout > 0 && data.workflowData.settings.executionTimeout;
        }
        if (workflowTimeout) {
            const timeout = Math.min(workflowTimeout, config.get('executions.maxTimeout')) * 1000;
            executionTimeout = setTimeout(() => {
                this.activeExecutions.stopExecution(executionId, 'timeout');
            }, timeout);
        }
        workflowExecution.then((fullRunData) => {
            clearTimeout(executionTimeout);
            if (workflowExecution.isCanceled) {
                fullRunData.finished = false;
            }
            this.activeExecutions.remove(executionId, fullRunData);
        });
        return executionId;
    }
    async runSubprocess(data, loadStaticData) {
        const startedAt = new Date();
        const subprocess = child_process_1.fork(path_1.join(__dirname, 'WorkflowRunnerProcess.js'));
        if (loadStaticData === true && data.workflowData.id) {
            data.workflowData.staticData = await _1.WorkflowHelpers.getStaticDataById(data.workflowData.id);
        }
        const executionId = this.activeExecutions.add(data, subprocess);
        let loadAllNodeTypes = false;
        for (const node of data.workflowData.nodes) {
            if (node.type === 'n8n-nodes-base.executeWorkflow') {
                loadAllNodeTypes = true;
                break;
            }
        }
        let nodeTypeData;
        let credentialTypeData;
        let credentialsOverwrites = this.credentialsOverwrites;
        if (loadAllNodeTypes === true) {
            nodeTypeData = _1.WorkflowHelpers.getAllNodeTypeData();
            const credentialTypes = _1.CredentialTypes();
            credentialTypeData = credentialTypes.credentialTypes;
        }
        else {
            nodeTypeData = _1.WorkflowHelpers.getNodeTypeData(data.workflowData.nodes);
            credentialTypeData = _1.WorkflowHelpers.getCredentialsData(data.credentials);
            credentialsOverwrites = {};
            for (const credentialName of Object.keys(credentialTypeData)) {
                if (this.credentialsOverwrites[credentialName] !== undefined) {
                    credentialsOverwrites[credentialName] = this.credentialsOverwrites[credentialName];
                }
            }
        }
        data.executionId = executionId;
        data.nodeTypeData = nodeTypeData;
        data.credentialsOverwrite = credentialsOverwrites;
        data.credentialsTypeData = credentialTypeData;
        const workflowHooks = _1.WorkflowExecuteAdditionalData.getWorkflowHooksMain(data, executionId);
        subprocess.send({ type: 'startWorkflow', data });
        let executionTimeout;
        let workflowTimeout = config.get('executions.timeout') > 0 && config.get('executions.timeout');
        if (data.workflowData.settings && data.workflowData.settings.executionTimeout) {
            workflowTimeout = data.workflowData.settings.executionTimeout > 0 && data.workflowData.settings.executionTimeout;
        }
        if (workflowTimeout) {
            const timeout = Math.min(workflowTimeout, config.get('executions.maxTimeout')) * 1000;
            executionTimeout = setTimeout(() => {
                this.activeExecutions.stopExecution(executionId, 'timeout');
                executionTimeout = setTimeout(() => subprocess.kill(), Math.max(timeout * 0.2, 5000));
            }, timeout);
        }
        subprocess.on('message', (message) => {
            if (message.type === 'end') {
                clearTimeout(executionTimeout);
                this.activeExecutions.remove(executionId, message.data.runData);
            }
            else if (message.type === 'processError') {
                clearTimeout(executionTimeout);
                const executionError = message.data.executionError;
                this.processError(executionError, startedAt, data.executionMode, executionId);
            }
            else if (message.type === 'processHook') {
                this.processHookMessage(workflowHooks, message.data);
            }
            else if (message.type === 'timeout') {
                const timeoutError = { message: 'Workflow execution timed out!' };
                this.processError(timeoutError, startedAt, data.executionMode, executionId);
            }
        });
        subprocess.on('exit', (code, signal) => {
            if (signal === 'SIGTERM') {
                const timeoutError = {
                    message: 'Workflow execution timed out!',
                };
                this.processError(timeoutError, startedAt, data.executionMode, executionId);
            }
            else if (code !== 0) {
                const executionError = {
                    message: 'Workflow execution process did crash for an unknown reason!',
                };
                this.processError(executionError, startedAt, data.executionMode, executionId);
            }
            clearTimeout(executionTimeout);
        });
        return executionId;
    }
}
exports.WorkflowRunner = WorkflowRunner;
//# sourceMappingURL=WorkflowRunner.js.map