"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Amqp = void 0;
class Amqp {
    constructor() {
        this.description = {
            displayName: 'AMQP Sender',
            name: 'amqp',
            icon: 'file:amqp.png',
            group: ['transform'],
            version: 1,
            description: 'Sends a raw-message via AMQP 1.0, executed once per item',
            defaults: {
                name: 'AMQP Sender',
                color: '#00FF00',
            },
            inputs: ['main'],
            outputs: ['main'],
            credentials: [{
                    name: 'amqp',
                    required: true,
                }],
            properties: [
                {
                    displayName: 'Queue / Topic',
                    name: 'sink',
                    type: 'string',
                    default: '',
                    placeholder: 'topic://sourcename.something',
                    description: 'name of the queue of topic to publish to',
                },
                {
                    displayName: 'Headers',
                    name: 'headerParametersJson',
                    type: 'json',
                    default: '',
                    description: 'Header parameters as JSON (flat object). Sent as application_properties in amqp-message meta info.',
                },
                {
                    displayName: 'Options',
                    name: 'options',
                    type: 'collection',
                    placeholder: 'Add Option',
                    default: {},
                    options: [
                        {
                            displayName: 'Data as Object',
                            name: 'dataAsObject',
                            type: 'boolean',
                            default: false,
                            description: 'Send the data as an object.',
                        },
                        {
                            displayName: 'Send property',
                            name: 'sendOnlyProperty',
                            type: 'string',
                            default: '',
                            description: 'The only property to send. If empty the whole item will be sent.',
                        },
                    ],
                },
            ],
        };
    }
    async executeSingle() {
        const item = this.getInputData();
        const credentials = this.getCredentials('amqp');
        if (!credentials) {
            throw new Error('Credentials are mandatory!');
        }
        const sink = this.getNodeParameter('sink', '');
        const applicationProperties = this.getNodeParameter('headerParametersJson', {});
        const options = this.getNodeParameter('options', {});
        let headerProperties = applicationProperties;
        if (typeof applicationProperties === 'string' && applicationProperties !== '') {
            headerProperties = JSON.parse(applicationProperties);
        }
        if (sink === '') {
            throw new Error('Queue or Topic required!');
        }
        const container = require('rhea');
        const connectOptions = {
            host: credentials.hostname,
            hostname: credentials.hostname,
            port: credentials.port,
            reconnect: true,
            reconnect_limit: 50,
        };
        if (credentials.username || credentials.password) {
            container.options.username = credentials.username;
            container.options.password = credentials.password;
            connectOptions.username = credentials.username;
            connectOptions.password = credentials.password;
        }
        if (credentials.transportType !== '') {
            connectOptions.transport = credentials.transportType;
        }
        const allSent = new Promise((resolve) => {
            container.on('sendable', (context) => {
                let body = item.json;
                const sendOnlyProperty = options.sendOnlyProperty;
                if (sendOnlyProperty) {
                    body = body[sendOnlyProperty];
                }
                if (options.dataAsObject !== true) {
                    body = JSON.stringify(body);
                }
                const message = {
                    application_properties: headerProperties,
                    body,
                };
                const sendResult = context.sender.send(message);
                resolve(sendResult);
            });
        });
        container.connect(connectOptions).open_sender(sink);
        const sendResult = await allSent;
        return { json: { id: sendResult.id } };
    }
}
exports.Amqp = Amqp;
//# sourceMappingURL=Amqp.node.js.map