"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AmqpTrigger = void 0;
class AmqpTrigger {
    constructor() {
        this.description = {
            displayName: 'AMQP Trigger',
            name: 'amqpTrigger',
            icon: 'file:amqp.png',
            group: ['trigger'],
            version: 1,
            description: 'Listens to AMQP 1.0 Messages',
            defaults: {
                name: 'AMQP Trigger',
                color: '#00FF00',
            },
            inputs: [],
            outputs: ['main'],
            credentials: [{
                    name: 'amqp',
                    required: true,
                }],
            properties: [
                {
                    displayName: 'Queue / Topic',
                    name: 'sink',
                    type: 'string',
                    default: '',
                    placeholder: 'topic://sourcename.something',
                    description: 'name of the queue of topic to listen to',
                },
                {
                    displayName: 'Clientname',
                    name: 'clientname',
                    type: 'string',
                    default: '',
                    placeholder: 'for durable/persistent topic subscriptions, example: "n8n"',
                    description: 'Leave empty for non-durable topic subscriptions or queues. ',
                },
                {
                    displayName: 'Subscription',
                    name: 'subscription',
                    type: 'string',
                    default: '',
                    placeholder: 'for durable/persistent topic subscriptions, example: "order-worker"',
                    description: 'Leave empty for non-durable topic subscriptions or queues',
                },
                {
                    displayName: 'Options',
                    name: 'options',
                    type: 'collection',
                    placeholder: 'Add Option',
                    default: {},
                    options: [
                        {
                            displayName: 'Convert Body To String',
                            name: 'jsonConvertByteArrayToString',
                            type: 'boolean',
                            default: false,
                            description: 'Convert JSON Body content (["body"]["content"]) from Byte Array to string. Needed for Azure Service Bus.',
                        },
                        {
                            displayName: 'JSON Parse Body',
                            name: 'jsonParseBody',
                            type: 'boolean',
                            default: false,
                            description: 'Parse the body to an object.',
                        },
                        {
                            displayName: 'Only Body',
                            name: 'onlyBody',
                            type: 'boolean',
                            default: false,
                            description: 'Returns only the body property.',
                        },
                    ],
                },
            ],
        };
    }
    async trigger() {
        const credentials = this.getCredentials('amqp');
        if (!credentials) {
            throw new Error('Credentials are mandatory!');
        }
        const sink = this.getNodeParameter('sink', '');
        const clientname = this.getNodeParameter('clientname', '');
        const subscription = this.getNodeParameter('subscription', '');
        const options = this.getNodeParameter('options', {});
        if (sink === '') {
            throw new Error('Queue or Topic required!');
        }
        let durable = false;
        if (subscription && clientname) {
            durable = true;
        }
        const container = require('rhea');
        const connectOptions = {
            host: credentials.hostname,
            hostname: credentials.hostname,
            port: credentials.port,
            reconnect: true,
            reconnect_limit: 50,
            container_id: (durable ? clientname : null),
        };
        if (credentials.username || credentials.password) {
            container.options.username = credentials.username;
            container.options.password = credentials.password;
            connectOptions.username = credentials.username;
            connectOptions.password = credentials.password;
        }
        if (credentials.transportType) {
            connectOptions.transport = credentials.transportType;
        }
        let lastMsgId = undefined;
        const self = this;
        container.on('message', (context) => {
            if (context.message.message_id && context.message.message_id === lastMsgId) {
                return;
            }
            lastMsgId = context.message.message_id;
            let data = context.message;
            if (options.jsonConvertByteArrayToString === true && data.body.content !== undefined) {
                const content = JSON.stringify(data.body.content);
                data.body = String.fromCharCode.apply(null, JSON.parse(content).data);
            }
            if (options.jsonParseBody === true) {
                data.body = JSON.parse(data.body);
            }
            if (options.onlyBody === true) {
                data = data.body;
            }
            self.emit([self.helpers.returnJsonArray([data])]);
        });
        const connection = container.connect(connectOptions);
        let clientOptions = undefined;
        if (durable) {
            clientOptions = {
                name: subscription,
                source: {
                    address: sink,
                    durable: 2,
                    expiry_policy: 'never',
                },
                credit_window: 1,
            };
        }
        else {
            clientOptions = {
                source: {
                    address: sink,
                },
                credit_window: 1,
            };
        }
        connection.open_receiver(clientOptions);
        async function closeFunction() {
            connection.close();
        }
        async function manualTriggerFunction() {
            await new Promise((resolve, reject) => {
                const timeoutHandler = setTimeout(() => {
                    reject(new Error('Aborted, no message received within 30secs. This 30sec timeout is only set for "manually triggered execution". Active Workflows will listen indefinitely.'));
                }, 30000);
                container.on('message', (context) => {
                    if (Object.keys(context.message)[0] === 'body' && Object.keys(context.message).length === 1) {
                        self.emit([self.helpers.returnJsonArray([context.message.body])]);
                    }
                    else {
                        self.emit([self.helpers.returnJsonArray([context.message])]);
                    }
                    clearTimeout(timeoutHandler);
                    resolve(true);
                });
            });
        }
        return {
            closeFunction,
            manualTriggerFunction,
        };
    }
}
exports.AmqpTrigger = AmqpTrigger;
//# sourceMappingURL=AmqpTrigger.node.js.map