"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getWorkspaces = exports.asanaApiRequestAllItems = exports.asanaApiRequest = void 0;
const lodash_1 = require("lodash");
async function asanaApiRequest(method, endpoint, body, query, uri) {
    const authenticationMethod = this.getNodeParameter('authentication', 0);
    const options = {
        headers: {},
        method,
        body: { data: body },
        qs: query,
        uri: uri || `https://app.asana.com/api/1.0${endpoint}`,
        json: true,
    };
    try {
        if (authenticationMethod === 'accessToken') {
            const credentials = this.getCredentials('asanaApi');
            if (credentials === undefined) {
                throw new Error('No credentials got returned!');
            }
            options.headers['Authorization'] = `Bearer ${credentials.accessToken}`;
            return await this.helpers.request(options);
        }
        else {
            return await this.helpers.requestOAuth2.call(this, 'asanaOAuth2Api', options);
        }
    }
    catch (error) {
        if (error.statusCode === 401) {
            throw new Error('The Asana credentials are not valid!');
        }
        if (error.statusCode === 403) {
            throw error;
        }
        if (error.response && error.response.body && error.response.body.errors) {
            const errorMessages = error.response.body.errors.map((errorData) => {
                return errorData.message;
            });
            throw new Error(`Asana error response [${error.statusCode}]: ${errorMessages.join(' | ')}`);
        }
        throw error;
    }
}
exports.asanaApiRequest = asanaApiRequest;
async function asanaApiRequestAllItems(method, endpoint, body = {}, query = {}) {
    const returnData = [];
    let responseData;
    let uri;
    query.limit = 100;
    do {
        responseData = await asanaApiRequest.call(this, method, endpoint, body, query, uri);
        uri = lodash_1.get(responseData, 'next_page.uri');
        returnData.push.apply(returnData, responseData['data']);
    } while (responseData['next_page'] !== null);
    return returnData;
}
exports.asanaApiRequestAllItems = asanaApiRequestAllItems;
async function getWorkspaces() {
    const endpoint = '/workspaces';
    const responseData = await asanaApiRequestAllItems.call(this, 'GET', endpoint, {});
    const returnData = [];
    for (const workspaceData of responseData) {
        if (workspaceData.resource_type !== 'workspace') {
            continue;
        }
        returnData.push({
            name: workspaceData.name,
            value: workspaceData.gid,
        });
    }
    returnData.sort((a, b) => {
        if (a.name < b.name) {
            return -1;
        }
        if (a.name > b.name) {
            return 1;
        }
        return 0;
    });
    return returnData;
}
exports.getWorkspaces = getWorkspaces;
//# sourceMappingURL=GenericFunctions.js.map