"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GitlabTrigger = void 0;
const GenericFunctions_1 = require("./GenericFunctions");
class GitlabTrigger {
    constructor() {
        this.description = {
            displayName: 'GitLab Trigger',
            name: 'gitlabTrigger',
            icon: 'file:gitlab.png',
            group: ['trigger'],
            version: 1,
            subtitle: '={{$parameter["owner"] + "/" + $parameter["repository"] + ": " + $parameter["events"].join(", ")}}',
            description: 'Starts the workflow when a GitLab event occurs.',
            defaults: {
                name: 'Gitlab Trigger',
                color: '#FC6D27',
            },
            inputs: [],
            outputs: ['main'],
            credentials: [
                {
                    name: 'gitlabApi',
                    required: true,
                    displayOptions: {
                        show: {
                            authentication: [
                                'accessToken',
                            ],
                        },
                    },
                },
                {
                    name: 'gitlabOAuth2Api',
                    required: true,
                    displayOptions: {
                        show: {
                            authentication: [
                                'oAuth2',
                            ],
                        },
                    },
                },
            ],
            webhooks: [
                {
                    name: 'default',
                    httpMethod: 'POST',
                    responseMode: 'onReceived',
                    path: 'webhook',
                },
            ],
            properties: [
                {
                    displayName: 'Authentication',
                    name: 'authentication',
                    type: 'options',
                    options: [
                        {
                            name: 'Access Token',
                            value: 'accessToken',
                        },
                        {
                            name: 'OAuth2',
                            value: 'oAuth2',
                        },
                    ],
                    default: 'accessToken',
                    description: 'The resource to operate on.',
                },
                {
                    displayName: 'Repository Owner',
                    name: 'owner',
                    type: 'string',
                    default: '',
                    required: true,
                    placeholder: 'n8n-io',
                    description: 'Owner of the repsitory.',
                },
                {
                    displayName: 'Repository Name',
                    name: 'repository',
                    type: 'string',
                    default: '',
                    required: true,
                    placeholder: 'n8n',
                    description: 'The name of the repsitory.',
                },
                {
                    displayName: 'Events',
                    name: 'events',
                    type: 'multiOptions',
                    options: [
                        {
                            name: '*',
                            value: '*',
                            description: 'Any time any event is triggered (Wildcard Event).',
                        },
                        {
                            name: 'Comment',
                            value: 'note',
                            description: 'Triggered when a new comment is made on commits, merge requests, issues, and code snippets.',
                        },
                        {
                            name: 'Issue',
                            value: 'issues',
                            description: 'Triggered when a new issue is created or an existing issue was updated/closed/reopened.',
                        },
                        {
                            name: 'Job',
                            value: 'job',
                            description: 'Triggered on status change of a job.',
                        },
                        {
                            name: 'Merge Request',
                            value: 'merge_requests',
                            description: 'Triggered when a new merge request is created, an existing merge request was updated/merged/closed or a commit is added in the source branch.',
                        },
                        {
                            name: 'Pipeline',
                            value: 'pipeline',
                            description: 'Triggered on status change of Pipeline.',
                        },
                        {
                            name: 'Push',
                            value: 'push',
                            description: 'Triggered when you push to the repository except when pushing tags.',
                        },
                        {
                            name: 'Tag',
                            value: 'tag_push',
                            description: 'Triggered when you create (or delete) tags to the repository.',
                        },
                        {
                            name: 'Wiki Page',
                            value: 'wiki_page',
                            description: 'Triggered when a wiki page is created, updated or deleted.',
                        },
                    ],
                    required: true,
                    default: [],
                    description: 'The events to listen to.',
                },
            ],
        };
        this.webhookMethods = {
            default: {
                async checkExists() {
                    const webhookData = this.getWorkflowStaticData('node');
                    if (webhookData.webhookId === undefined) {
                        return false;
                    }
                    const owner = this.getNodeParameter('owner');
                    const repository = this.getNodeParameter('repository');
                    const path = (`${owner}/${repository}`).replace(/\//g, '%2F');
                    const endpoint = `/projects/${path}/hooks/${webhookData.webhookId}`;
                    try {
                        await GenericFunctions_1.gitlabApiRequest.call(this, 'GET', endpoint, {});
                    }
                    catch (e) {
                        if (e.message.includes('[404]:')) {
                            delete webhookData.webhookId;
                            delete webhookData.webhookEvents;
                            return false;
                        }
                        throw e;
                    }
                    return true;
                },
                async create() {
                    const webhookUrl = this.getNodeWebhookUrl('default');
                    const owner = this.getNodeParameter('owner');
                    const repository = this.getNodeParameter('repository');
                    let eventsArray = this.getNodeParameter('events', []);
                    if (eventsArray.includes('*')) {
                        eventsArray = ['note', 'issues', 'job', 'merge_requests', 'pipeline', 'push', 'tag_push', 'wiki_page'];
                    }
                    const events = {};
                    for (const e of eventsArray) {
                        events[`${e}_events`] = true;
                    }
                    if (events['push_events'] === undefined) {
                        events['push_events'] = false;
                    }
                    const path = (`${owner}/${repository}`).replace(/\//g, '%2F');
                    const endpoint = `/projects/${path}/hooks`;
                    const body = Object.assign(Object.assign({ url: webhookUrl }, events), { enable_ssl_verification: false });
                    let responseData;
                    try {
                        responseData = await GenericFunctions_1.gitlabApiRequest.call(this, 'POST', endpoint, body);
                    }
                    catch (e) {
                        throw e;
                    }
                    if (responseData.id === undefined) {
                        throw new Error('GitLab webhook creation response did not contain the expected data.');
                    }
                    const webhookData = this.getWorkflowStaticData('node');
                    webhookData.webhookId = responseData.id;
                    webhookData.webhookEvents = eventsArray;
                    return true;
                },
                async delete() {
                    const webhookData = this.getWorkflowStaticData('node');
                    if (webhookData.webhookId !== undefined) {
                        const owner = this.getNodeParameter('owner');
                        const repository = this.getNodeParameter('repository');
                        const path = (`${owner}/${repository}`).replace(/\//g, '%2F');
                        const endpoint = `/projects/${path}/hooks/${webhookData.webhookId}`;
                        const body = {};
                        try {
                            await GenericFunctions_1.gitlabApiRequest.call(this, 'DELETE', endpoint, body);
                        }
                        catch (e) {
                            return false;
                        }
                        delete webhookData.webhookId;
                        delete webhookData.webhookEvents;
                    }
                    return true;
                },
            },
        };
    }
    async webhook() {
        const bodyData = this.getBodyData();
        const returnData = [];
        returnData.push({
            body: bodyData,
            headers: this.getHeaderData(),
            query: this.getQueryData(),
        });
        return {
            workflowData: [
                this.helpers.returnJsonArray(returnData),
            ],
        };
    }
}
exports.GitlabTrigger = GitlabTrigger;
//# sourceMappingURL=GitlabTrigger.node.js.map