"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HubspotTrigger = void 0;
const GenericFunctions_1 = require("./GenericFunctions");
const crypto_1 = require("crypto");
class HubspotTrigger {
    constructor() {
        this.description = {
            displayName: 'HubSpot Trigger',
            name: 'hubspotTrigger',
            icon: 'file:hubspot.png',
            group: ['trigger'],
            version: 1,
            subtitle: '={{($parameter["appId"]) ? $parameter["event"] : ""}}',
            description: 'Starts the workflow when HubSpot events occur.',
            defaults: {
                name: 'Hubspot Trigger',
                color: '#ff7f64',
            },
            inputs: [],
            outputs: ['main'],
            credentials: [
                {
                    name: 'hubspotDeveloperApi',
                    required: true,
                },
            ],
            webhooks: [
                {
                    name: 'default',
                    httpMethod: 'POST',
                    responseMode: 'onReceived',
                    path: 'webhook',
                },
                {
                    name: 'setup',
                    httpMethod: 'GET',
                    responseMode: 'onReceived',
                    path: 'webhook',
                },
            ],
            properties: [
                {
                    displayName: 'App ID',
                    name: 'appId',
                    type: 'string',
                    default: '',
                    required: true,
                    description: 'App ID',
                },
                {
                    displayName: 'Event',
                    name: 'event',
                    type: 'options',
                    options: [
                        {
                            name: 'contact.creation',
                            value: 'contact.creation',
                            description: `To get notified if any contact is created in a customer's account.`,
                        },
                        {
                            name: 'contact.deletion',
                            value: 'contact.deletion',
                            description: `To get notified if any contact is deleted in a customer's account.`,
                        },
                        {
                            name: 'contact.privacyDeletion',
                            value: 'contact.privacyDeletion',
                            description: `To get notified if a contact is deleted for privacy compliance reasons. `,
                        },
                        {
                            name: 'contact.propertyChange',
                            value: 'contact.propertyChange',
                            description: `to get notified if a specified property is changed for any contact in a customer's account. `,
                        },
                        {
                            name: 'company.creation',
                            value: 'company.creation',
                            description: `To get notified if any company is created in a customer's account.`,
                        },
                        {
                            name: 'company.deletion',
                            value: 'company.deletion',
                            description: `To get notified if any company is deleted in a customer's account.`,
                        },
                        {
                            name: 'company.propertyChange',
                            value: 'company.propertyChange',
                            description: `To get notified if a specified property is changed for any company in a customer's account.`,
                        },
                        {
                            name: 'deal.creation',
                            value: 'deal.creation',
                            description: `To get notified if any deal is created in a customer's account.`,
                        },
                        {
                            name: 'deal.deletion',
                            value: 'deal.deletion',
                            description: `To get notified if any deal is deleted in a customer's account.`,
                        },
                        {
                            name: 'deal.propertyChange',
                            value: 'deal.propertyChange',
                            description: `To get notified if a specified property is changed for any deal in a customer's account.`,
                        },
                    ],
                    default: 'contact.creation',
                    required: true,
                },
                {
                    displayName: 'Property',
                    name: 'property',
                    type: 'string',
                    displayOptions: {
                        show: {
                            event: [
                                'contact.propertyChange',
                                'company.propertyChange',
                                'deal.propertyChange',
                            ],
                        },
                    },
                    default: '',
                    required: true,
                },
                {
                    displayName: 'Additional Fields',
                    name: 'additionalFields',
                    type: 'collection',
                    placeholder: 'Add Field',
                    default: {},
                    options: [
                        {
                            displayName: 'Max Concurrent Requests',
                            name: 'maxConcurrentRequests',
                            type: 'number',
                            typeOptions: {
                                minValue: 5,
                            },
                            default: 5,
                        },
                    ],
                },
            ],
        };
        this.webhookMethods = {
            default: {
                async checkExists() {
                    const app = parseInt(this.getNodeParameter('appId'), 10);
                    const event = this.getNodeParameter('event');
                    const webhookUrlUi = this.getNodeWebhookUrl('default');
                    let endpoint = `/webhooks/v1/${app}/settings`;
                    const { webhookUrl, appId } = await GenericFunctions_1.hubspotApiRequest.call(this, 'GET', endpoint, {});
                    endpoint = `/webhooks/v1/${app}/subscriptions`;
                    const subscriptions = await GenericFunctions_1.hubspotApiRequest.call(this, 'GET', endpoint, {});
                    for (const subscription of subscriptions) {
                        if (webhookUrl === webhookUrlUi
                            && appId === app
                            && subscription.subscriptionDetails.subscriptionType === event
                            && subscription.enabled === true) {
                            return true;
                        }
                    }
                    return false;
                },
                async create() {
                    const webhookUrl = this.getNodeWebhookUrl('default');
                    const app = this.getNodeParameter('appId');
                    const event = this.getNodeParameter('event');
                    const additionalFields = this.getNodeParameter('additionalFields');
                    const propertyEvents = [
                        'contact.propertyChange',
                        'company.propertyChange',
                        'deal.propertyChange',
                    ];
                    let endpoint = `/webhooks/v1/${app}/settings`;
                    let body = {
                        webhookUrl,
                        maxConcurrentRequests: additionalFields.maxConcurrentRequests || 5,
                    };
                    await GenericFunctions_1.hubspotApiRequest.call(this, 'PUT', endpoint, body);
                    endpoint = `/webhooks/v1/${app}/subscriptions`;
                    body = {
                        subscriptionDetails: {
                            subscriptionType: event,
                        },
                        enabled: true,
                    };
                    if (propertyEvents.includes(event)) {
                        const property = this.getNodeParameter('property');
                        body.subscriptionDetails.propertyName = property;
                    }
                    const responseData = await GenericFunctions_1.hubspotApiRequest.call(this, 'POST', endpoint, body);
                    if (responseData.id === undefined) {
                        return false;
                    }
                    const webhookData = this.getWorkflowStaticData('node');
                    webhookData.webhookId = responseData.id;
                    return true;
                },
                async delete() {
                    const webhookData = this.getWorkflowStaticData('node');
                    const app = this.getNodeParameter('appId');
                    if (webhookData.webhookId !== undefined) {
                        const endpoint = `/webhooks/v1/${app}/subscriptions/${webhookData.webhookId}`;
                        const body = {};
                        try {
                            await GenericFunctions_1.hubspotApiRequest.call(this, 'DELETE', endpoint, body);
                        }
                        catch (e) {
                            return false;
                        }
                        delete webhookData.webhookId;
                    }
                    return true;
                },
            },
        };
    }
    async webhook() {
        const credentials = this.getCredentials('hubspotDeveloperApi');
        if (credentials === undefined) {
            throw new Error('No credentials found!');
        }
        const req = this.getRequestObject();
        const bodyData = req.body;
        const headerData = this.getHeaderData();
        if (headerData['x-hubspot-signature'] === undefined) {
            return {};
        }
        if (credentials.clientSecret !== '') {
            const hash = `${credentials.clientSecret}${JSON.stringify(bodyData)}`;
            const signature = crypto_1.createHash('sha256').update(hash).digest('hex');
            if (signature !== headerData['x-hubspot-signature']) {
                return {};
            }
        }
        for (let i = 0; i < bodyData.length; i++) {
            const subscriptionType = bodyData[i].subscriptionType;
            if (subscriptionType.includes('contact')) {
                bodyData[i].contactId = bodyData[i].objectId;
            }
            if (subscriptionType.includes('company')) {
                bodyData[i].companyId = bodyData[i].objectId;
            }
            if (subscriptionType.includes('deal')) {
                bodyData[i].dealId = bodyData[i].objectId;
            }
            delete bodyData[i].objectId;
        }
        return {
            workflowData: [
                this.helpers.returnJsonArray(bodyData),
            ],
        };
    }
}
exports.HubspotTrigger = HubspotTrigger;
//# sourceMappingURL=HubspotTrigger.node.js.map