"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Jira = void 0;
const GenericFunctions_1 = require("./GenericFunctions");
const IssueDescription_1 = require("./IssueDescription");
class Jira {
    constructor() {
        this.description = {
            displayName: 'Jira Software',
            name: 'jira',
            icon: 'file:jira.png',
            group: ['output'],
            version: 1,
            subtitle: '={{$parameter["operation"] + ": " + $parameter["resource"]}}',
            description: 'Consume Jira Software API',
            defaults: {
                name: 'Jira',
                color: '#4185f7',
            },
            inputs: ['main'],
            outputs: ['main'],
            credentials: [
                {
                    name: 'jiraSoftwareCloudApi',
                    required: true,
                    displayOptions: {
                        show: {
                            jiraVersion: [
                                'cloud',
                            ],
                        },
                    },
                },
                {
                    name: 'jiraSoftwareServerApi',
                    required: true,
                    displayOptions: {
                        show: {
                            jiraVersion: [
                                'server',
                            ],
                        },
                    },
                },
            ],
            properties: [
                {
                    displayName: 'Jira Version',
                    name: 'jiraVersion',
                    type: 'options',
                    options: [
                        {
                            name: 'Cloud',
                            value: 'cloud',
                        },
                        {
                            name: 'Server (Self Hosted)',
                            value: 'server',
                        },
                    ],
                    default: 'cloud',
                },
                {
                    displayName: 'Resource',
                    name: 'resource',
                    type: 'options',
                    options: [
                        {
                            name: 'Issue',
                            value: 'issue',
                            description: 'Creates an issue or, where the option to create subtasks is enabled in Jira, a subtask',
                        },
                    ],
                    default: 'issue',
                    description: 'Resource to consume.',
                },
                ...IssueDescription_1.issueOperations,
                ...IssueDescription_1.issueFields,
            ],
        };
        this.methods = {
            loadOptions: {
                async getProjects() {
                    const returnData = [];
                    const jiraVersion = this.getCurrentNodeParameter('jiraVersion');
                    let endpoint = '/api/2/project/search';
                    if (jiraVersion === 'server') {
                        endpoint = '/api/2/project';
                    }
                    let projects = await GenericFunctions_1.jiraSoftwareCloudApiRequest.call(this, endpoint, 'GET');
                    if (projects.values && Array.isArray(projects.values)) {
                        projects = projects.values;
                    }
                    for (const project of projects) {
                        const projectName = project.name;
                        const projectId = project.id;
                        returnData.push({
                            name: projectName,
                            value: projectId,
                        });
                    }
                    return returnData;
                },
                async getIssueTypes() {
                    const projectId = this.getCurrentNodeParameter('project');
                    const returnData = [];
                    const issueTypes = await GenericFunctions_1.jiraSoftwareCloudApiRequest.call(this, '/api/2/issuetype', 'GET');
                    const jiraVersion = this.getCurrentNodeParameter('jiraVersion');
                    if (jiraVersion === 'server') {
                        for (const issueType of issueTypes) {
                            const issueTypeName = issueType.name;
                            const issueTypeId = issueType.id;
                            returnData.push({
                                name: issueTypeName,
                                value: issueTypeId,
                            });
                        }
                    }
                    else {
                        for (const issueType of issueTypes) {
                            if (issueType.scope === undefined || issueType.scope.project.id === projectId) {
                                const issueTypeName = issueType.name;
                                const issueTypeId = issueType.id;
                                returnData.push({
                                    name: issueTypeName,
                                    value: issueTypeId,
                                });
                            }
                        }
                    }
                    return returnData;
                },
                async getLabels() {
                    const returnData = [];
                    const labels = await GenericFunctions_1.jiraSoftwareCloudApiRequest.call(this, '/api/2/label', 'GET');
                    for (const label of labels.values) {
                        const labelName = label;
                        const labelId = label;
                        returnData.push({
                            name: labelName,
                            value: labelId,
                        });
                    }
                    return returnData;
                },
                async getPriorities() {
                    const returnData = [];
                    const priorities = await GenericFunctions_1.jiraSoftwareCloudApiRequest.call(this, '/api/2/priority', 'GET');
                    for (const priority of priorities) {
                        const priorityName = priority.name;
                        const priorityId = priority.id;
                        returnData.push({
                            name: priorityName,
                            value: priorityId,
                        });
                    }
                    return returnData;
                },
                async getUsers() {
                    const returnData = [];
                    const jiraVersion = this.getCurrentNodeParameter('jiraVersion');
                    if (jiraVersion === 'server') {
                        const users = await GenericFunctions_1.jiraSoftwareCloudApiRequest.call(this, '/api/2/user/search', 'GET', {}, {
                            username: "'",
                        });
                        for (const user of users) {
                            const userName = user.displayName;
                            const userId = user.name;
                            returnData.push({
                                name: userName,
                                value: userId,
                            });
                        }
                    }
                    else {
                        const users = await GenericFunctions_1.jiraSoftwareCloudApiRequest.call(this, '/api/2/users/search', 'GET');
                        for (const user of users) {
                            const userName = user.displayName;
                            const userId = user.accountId;
                            returnData.push({
                                name: userName,
                                value: userId,
                            });
                        }
                    }
                    return returnData;
                },
                async getGroups() {
                    const returnData = [];
                    const groups = await GenericFunctions_1.jiraSoftwareCloudApiRequest.call(this, '/api/2/groups/picker', 'GET');
                    for (const group of groups.groups) {
                        const groupName = group.name;
                        const groupId = group.name;
                        returnData.push({
                            name: groupName,
                            value: groupId,
                        });
                    }
                    return returnData;
                },
                async getTransitions() {
                    const returnData = [];
                    const issueKey = this.getCurrentNodeParameter('issueKey');
                    const transitions = await GenericFunctions_1.jiraSoftwareCloudApiRequest.call(this, `/api/2/issue/${issueKey}/transitions`, 'GET');
                    for (const transition of transitions.transitions) {
                        returnData.push({
                            name: transition.name,
                            value: transition.id,
                        });
                    }
                    return returnData;
                },
            },
        };
    }
    async execute() {
        const items = this.getInputData();
        const returnData = [];
        const length = items.length;
        let responseData;
        const qs = {};
        const resource = this.getNodeParameter('resource', 0);
        const operation = this.getNodeParameter('operation', 0);
        const jiraVersion = this.getNodeParameter('jiraVersion', 0);
        for (let i = 0; i < length; i++) {
            if (resource === 'issue') {
                if (operation === 'create') {
                    const summary = this.getNodeParameter('summary', i);
                    const projectId = this.getNodeParameter('project', i);
                    const issueTypeId = this.getNodeParameter('issueType', i);
                    const additionalFields = this.getNodeParameter('additionalFields', i);
                    const body = {};
                    const fields = {
                        summary,
                        project: {
                            id: projectId,
                        },
                        issuetype: {
                            id: issueTypeId,
                        },
                    };
                    if (additionalFields.labels) {
                        fields.labels = additionalFields.labels;
                    }
                    if (additionalFields.serverLabels) {
                        fields.labels = additionalFields.serverLabels;
                    }
                    if (additionalFields.priority) {
                        fields.priority = {
                            id: additionalFields.priority,
                        };
                    }
                    if (additionalFields.assignee) {
                        if (jiraVersion === 'server') {
                            fields.assignee = {
                                name: additionalFields.assignee,
                            };
                        }
                        else {
                            fields.assignee = {
                                id: additionalFields.assignee,
                            };
                        }
                    }
                    if (additionalFields.description) {
                        fields.description = additionalFields.description;
                    }
                    if (additionalFields.updateHistory) {
                        qs.updateHistory = additionalFields.updateHistory;
                    }
                    const issueTypes = await GenericFunctions_1.jiraSoftwareCloudApiRequest.call(this, '/api/2/issuetype', 'GET', body, qs);
                    const subtaskIssues = [];
                    for (const issueType of issueTypes) {
                        if (issueType.subtask) {
                            subtaskIssues.push(issueType.id);
                        }
                    }
                    if (!additionalFields.parentIssueKey
                        && subtaskIssues.includes(issueTypeId)) {
                        throw new Error('You must define a Parent Issue Key when Issue type is sub-task');
                    }
                    else if (additionalFields.parentIssueKey
                        && subtaskIssues.includes(issueTypeId)) {
                        fields.parent = {
                            key: additionalFields.parentIssueKey.toUpperCase(),
                        };
                    }
                    body.fields = fields;
                    responseData = await GenericFunctions_1.jiraSoftwareCloudApiRequest.call(this, '/api/2/issue', 'POST', body);
                }
                if (operation === 'update') {
                    const issueKey = this.getNodeParameter('issueKey', i);
                    const updateFields = this.getNodeParameter('updateFields', i);
                    const body = {};
                    const fields = {};
                    if (updateFields.summary) {
                        fields.summary = updateFields.summary;
                    }
                    if (updateFields.issueType) {
                        fields.issuetype = {
                            id: updateFields.issueType,
                        };
                    }
                    if (updateFields.labels) {
                        fields.labels = updateFields.labels;
                    }
                    if (updateFields.serverLabels) {
                        fields.labels = updateFields.serverLabels;
                    }
                    if (updateFields.priority) {
                        fields.priority = {
                            id: updateFields.priority,
                        };
                    }
                    if (updateFields.assignee) {
                        if (jiraVersion === 'server') {
                            fields.assignee = {
                                name: updateFields.assignee,
                            };
                        }
                        else {
                            fields.assignee = {
                                id: updateFields.assignee,
                            };
                        }
                    }
                    if (updateFields.description) {
                        fields.description = updateFields.description;
                    }
                    const issueTypes = await GenericFunctions_1.jiraSoftwareCloudApiRequest.call(this, '/api/2/issuetype', 'GET', body);
                    const subtaskIssues = [];
                    for (const issueType of issueTypes) {
                        if (issueType.subtask) {
                            subtaskIssues.push(issueType.id);
                        }
                    }
                    if (!updateFields.parentIssueKey
                        && subtaskIssues.includes(updateFields.issueType)) {
                        throw new Error('You must define a Parent Issue Key when Issue type is sub-task');
                    }
                    else if (updateFields.parentIssueKey
                        && subtaskIssues.includes(updateFields.issueType)) {
                        fields.parent = {
                            key: updateFields.parentIssueKey.toUpperCase(),
                        };
                    }
                    body.fields = fields;
                    if (updateFields.statusId) {
                        responseData = await GenericFunctions_1.jiraSoftwareCloudApiRequest.call(this, `/api/2/issue/${issueKey}/transitions`, 'POST', { transition: { id: updateFields.statusId } });
                    }
                    responseData = await GenericFunctions_1.jiraSoftwareCloudApiRequest.call(this, `/api/2/issue/${issueKey}`, 'PUT', body);
                    responseData = { success: true };
                }
                if (operation === 'get') {
                    const issueKey = this.getNodeParameter('issueKey', i);
                    const additionalFields = this.getNodeParameter('additionalFields', i);
                    if (additionalFields.fields) {
                        qs.fields = additionalFields.fields;
                    }
                    if (additionalFields.fieldsByKey) {
                        qs.fieldsByKey = additionalFields.fieldsByKey;
                    }
                    if (additionalFields.expand) {
                        qs.expand = additionalFields.expand;
                    }
                    if (additionalFields.properties) {
                        qs.properties = additionalFields.properties;
                    }
                    if (additionalFields.updateHistory) {
                        qs.updateHistory = additionalFields.updateHistory;
                    }
                    responseData = await GenericFunctions_1.jiraSoftwareCloudApiRequest.call(this, `/api/2/issue/${issueKey}`, 'GET', {}, qs);
                }
                if (operation === 'getAll') {
                    const returnAll = this.getNodeParameter('returnAll', i);
                    const options = this.getNodeParameter('options', i);
                    const body = {};
                    if (options.fields) {
                        body.fields = options.fields.split(',');
                    }
                    if (options.jql) {
                        body.jql = options.jql;
                    }
                    if (options.expand) {
                        body.expand = options.expand;
                    }
                    if (returnAll) {
                        responseData = await GenericFunctions_1.jiraSoftwareCloudApiRequestAllItems.call(this, 'issues', `/api/2/search`, 'POST', body);
                    }
                    else {
                        const limit = this.getNodeParameter('limit', i);
                        body.maxResults = limit;
                        responseData = await GenericFunctions_1.jiraSoftwareCloudApiRequest.call(this, `/api/2/search`, 'POST', body);
                        responseData = responseData.issues;
                    }
                }
                if (operation === 'changelog') {
                    const issueKey = this.getNodeParameter('issueKey', i);
                    const returnAll = this.getNodeParameter('returnAll', i);
                    if (returnAll) {
                        responseData = await GenericFunctions_1.jiraSoftwareCloudApiRequestAllItems.call(this, 'values', `/api/2/issue/${issueKey}/changelog`, 'GET');
                    }
                    else {
                        qs.maxResults = this.getNodeParameter('limit', i);
                        responseData = await GenericFunctions_1.jiraSoftwareCloudApiRequest.call(this, `/api/2/issue/${issueKey}/changelog`, 'GET', {}, qs);
                        responseData = responseData.values;
                    }
                }
                if (operation === 'notify') {
                    const issueKey = this.getNodeParameter('issueKey', i);
                    const additionalFields = this.getNodeParameter('additionalFields', i);
                    const jsonActive = this.getNodeParameter('jsonParameters', 0);
                    const body = {};
                    if (additionalFields.textBody) {
                        body.textBody = additionalFields.textBody;
                    }
                    if (additionalFields.htmlBody) {
                        body.htmlBody = additionalFields.htmlBody;
                    }
                    if (!jsonActive) {
                        const notificationRecipientsValues = this.getNodeParameter('notificationRecipientsUi', i).notificationRecipientsValues;
                        const notificationRecipients = {};
                        if (notificationRecipientsValues) {
                            if (notificationRecipientsValues.reporter) {
                                notificationRecipients.reporter = notificationRecipientsValues.reporter;
                            }
                            if (notificationRecipientsValues.assignee) {
                                notificationRecipients.assignee = notificationRecipientsValues.assignee;
                            }
                            if (notificationRecipientsValues.assignee) {
                                notificationRecipients.watchers = notificationRecipientsValues.watchers;
                            }
                            if (notificationRecipientsValues.voters) {
                                notificationRecipients.watchers = notificationRecipientsValues.voters;
                            }
                            if (notificationRecipientsValues.users.length > 0) {
                                notificationRecipients.users = notificationRecipientsValues.users.map(user => {
                                    return {
                                        accountId: user,
                                    };
                                });
                            }
                            if (notificationRecipientsValues.groups.length > 0) {
                                notificationRecipients.groups = notificationRecipientsValues.groups.map(group => {
                                    return {
                                        name: group,
                                    };
                                });
                            }
                        }
                        body.to = notificationRecipients;
                        const notificationRecipientsRestrictionsValues = this.getNodeParameter('notificationRecipientsRestrictionsUi', i).notificationRecipientsRestrictionsValues;
                        const notificationRecipientsRestrictions = {};
                        if (notificationRecipientsRestrictionsValues) {
                            if (notificationRecipientsRestrictionsValues.groups.length > 0) {
                                notificationRecipientsRestrictions.groups = notificationRecipientsRestrictionsValues.groups.map(group => {
                                    return {
                                        name: group,
                                    };
                                });
                            }
                        }
                        body.restrict = notificationRecipientsRestrictions;
                    }
                    else {
                        const notificationRecipientsJson = GenericFunctions_1.validateJSON(this.getNodeParameter('notificationRecipientsJson', i));
                        if (notificationRecipientsJson) {
                            body.to = notificationRecipientsJson;
                        }
                        const notificationRecipientsRestrictionsJson = GenericFunctions_1.validateJSON(this.getNodeParameter('notificationRecipientsRestrictionsJson', i));
                        if (notificationRecipientsRestrictionsJson) {
                            body.restrict = notificationRecipientsRestrictionsJson;
                        }
                    }
                    responseData = await GenericFunctions_1.jiraSoftwareCloudApiRequest.call(this, `/api/2/issue/${issueKey}/notify`, 'POST', body, qs);
                }
                if (operation === 'transitions') {
                    const issueKey = this.getNodeParameter('issueKey', i);
                    const additionalFields = this.getNodeParameter('additionalFields', i);
                    if (additionalFields.transitionId) {
                        qs.transitionId = additionalFields.transitionId;
                    }
                    if (additionalFields.expand) {
                        qs.expand = additionalFields.expand;
                    }
                    if (additionalFields.skipRemoteOnlyCondition) {
                        qs.skipRemoteOnlyCondition = additionalFields.skipRemoteOnlyCondition;
                    }
                    responseData = await GenericFunctions_1.jiraSoftwareCloudApiRequest.call(this, `/api/2/issue/${issueKey}/transitions`, 'GET', {}, qs);
                    responseData = responseData.transitions;
                }
                if (operation === 'delete') {
                    const issueKey = this.getNodeParameter('issueKey', i);
                    const deleteSubtasks = this.getNodeParameter('deleteSubtasks', i);
                    qs.deleteSubtasks = deleteSubtasks;
                    responseData = await GenericFunctions_1.jiraSoftwareCloudApiRequest.call(this, `/api/2/issue/${issueKey}`, 'DELETE', {}, qs);
                }
            }
            if (Array.isArray(responseData)) {
                returnData.push.apply(returnData, responseData);
            }
            else {
                returnData.push(responseData);
            }
        }
        return [this.helpers.returnJsonArray(returnData)];
    }
}
exports.Jira = Jira;
//# sourceMappingURL=Jira.node.js.map