"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MqttTrigger = void 0;
const mqtt = require("mqtt");
class MqttTrigger {
    constructor() {
        this.description = {
            displayName: 'MQTT Trigger',
            name: 'mqttTrigger',
            icon: 'file:mqtt.png',
            group: ['trigger'],
            version: 1,
            description: 'Listens to MQTT events',
            defaults: {
                name: 'MQTT Trigger',
                color: '#9b27af',
            },
            inputs: [],
            outputs: ['main'],
            credentials: [
                {
                    name: 'mqtt',
                    required: true,
                },
            ],
            properties: [
                {
                    displayName: 'Topics',
                    name: 'topics',
                    type: 'string',
                    default: '',
                    description: `Topics to subscribe to, multiple can be defined with comma.<br/>
				wildcard characters are supported (+ - for single level and # - for multi level)`,
                },
                {
                    displayName: 'Options',
                    name: 'options',
                    type: 'collection',
                    placeholder: 'Add Option',
                    default: {},
                    options: [
                        {
                            displayName: 'Only Message',
                            name: 'onlyMessage',
                            type: 'boolean',
                            default: false,
                            description: 'Returns only the message property.',
                        },
                        {
                            displayName: 'JSON Parse Message',
                            name: 'jsonParseMessage',
                            type: 'boolean',
                            default: false,
                            description: 'Try to parse the message to an object.',
                        },
                    ],
                },
            ],
        };
    }
    async trigger() {
        const credentials = this.getCredentials('mqtt');
        if (!credentials) {
            throw new Error('Credentials are mandatory!');
        }
        const topics = this.getNodeParameter('topics').split(',');
        const options = this.getNodeParameter('options');
        if (!topics) {
            throw new Error('Topics are mandatory!');
        }
        const protocol = credentials.protocol || 'mqtt';
        const host = credentials.host;
        const brokerUrl = `${protocol}://${host}`;
        const port = credentials.port || 1883;
        const clientOptions = {
            port,
        };
        if (credentials.username && credentials.password) {
            clientOptions.username = credentials.username;
            clientOptions.password = credentials.password;
        }
        const client = mqtt.connect(brokerUrl, clientOptions);
        const self = this;
        async function manualTriggerFunction() {
            await new Promise((resolve, reject) => {
                client.on('connect', () => {
                    client.subscribe(topics, (err, granted) => {
                        if (err) {
                            reject(err);
                        }
                        client.on('message', (topic, message) => {
                            let result = {};
                            message = message.toString();
                            if (options.jsonParseMessage) {
                                try {
                                    message = JSON.parse(message.toString());
                                }
                                catch (err) { }
                            }
                            result.message = message;
                            result.topic = topic;
                            if (options.onlyMessage) {
                                result = message;
                            }
                            self.emit([self.helpers.returnJsonArray([result])]);
                            resolve(true);
                        });
                    });
                });
                client.on('error', (error) => {
                    reject(error);
                });
            });
        }
        manualTriggerFunction();
        async function closeFunction() {
            client.end();
        }
        return {
            closeFunction,
            manualTriggerFunction,
        };
    }
}
exports.MqttTrigger = MqttTrigger;
//# sourceMappingURL=MqttTrigger.node.js.map