"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MicrosoftTeams = void 0;
const GenericFunctions_1 = require("./GenericFunctions");
const ChannelDescription_1 = require("./ChannelDescription");
const ChannelMessageDescription_1 = require("./ChannelMessageDescription");
class MicrosoftTeams {
    constructor() {
        this.description = {
            displayName: 'Microsoft Teams',
            name: 'microsoftTeams',
            icon: 'file:teams.png',
            group: ['input'],
            version: 1,
            subtitle: '={{$parameter["operation"] + ": " + $parameter["resource"]}}',
            description: 'Consume Microsoft Teams API',
            defaults: {
                name: 'Microsoft Teams',
                color: '#555cc7',
            },
            inputs: ['main'],
            outputs: ['main'],
            credentials: [
                {
                    name: 'microsoftTeamsOAuth2Api',
                    required: true,
                },
            ],
            properties: [
                {
                    displayName: 'Resource',
                    name: 'resource',
                    type: 'options',
                    options: [
                        {
                            name: 'Channel',
                            value: 'channel',
                        },
                        {
                            name: 'Channel Message (Beta)',
                            value: 'channelMessage',
                        },
                    ],
                    default: 'channel',
                    description: 'The resource to operate on.',
                },
                ...ChannelDescription_1.channelOperations,
                ...ChannelDescription_1.channelFields,
                ...ChannelMessageDescription_1.channelMessageOperations,
                ...ChannelMessageDescription_1.channelMessageFields,
            ],
        };
        this.methods = {
            loadOptions: {
                async getChannels() {
                    const returnData = [];
                    const teamId = this.getCurrentNodeParameter('teamId');
                    const { value } = await GenericFunctions_1.microsoftApiRequest.call(this, 'GET', `/v1.0/teams/${teamId}/channels`);
                    for (const channel of value) {
                        const channelName = channel.displayName;
                        const channelId = channel.id;
                        returnData.push({
                            name: channelName,
                            value: channelId,
                        });
                    }
                    return returnData;
                },
                async getTeams() {
                    const returnData = [];
                    const { value } = await GenericFunctions_1.microsoftApiRequest.call(this, 'GET', '/v1.0/me/joinedTeams');
                    for (const team of value) {
                        const teamName = team.displayName;
                        const teamId = team.id;
                        returnData.push({
                            name: teamName,
                            value: teamId,
                        });
                    }
                    return returnData;
                },
            },
        };
    }
    async execute() {
        const items = this.getInputData();
        const returnData = [];
        const length = items.length;
        const qs = {};
        let responseData;
        const resource = this.getNodeParameter('resource', 0);
        const operation = this.getNodeParameter('operation', 0);
        for (let i = 0; i < length; i++) {
            if (resource === 'channel') {
                if (operation === 'create') {
                    const teamId = this.getNodeParameter('teamId', i);
                    const name = this.getNodeParameter('name', i);
                    const options = this.getNodeParameter('options', i);
                    const body = {
                        displayName: name,
                    };
                    if (options.description) {
                        body.description = options.description;
                    }
                    if (options.type) {
                        body.membershipType = options.type;
                    }
                    responseData = await GenericFunctions_1.microsoftApiRequest.call(this, 'POST', `/v1.0/teams/${teamId}/channels`, body);
                }
                if (operation === 'delete') {
                    const teamId = this.getNodeParameter('teamId', i);
                    const channelId = this.getNodeParameter('channelId', i);
                    responseData = await GenericFunctions_1.microsoftApiRequest.call(this, 'DELETE', `/v1.0/teams/${teamId}/channels/${channelId}`);
                    responseData = { success: true };
                }
                if (operation === 'get') {
                    const teamId = this.getNodeParameter('teamId', i);
                    const channelId = this.getNodeParameter('channelId', i);
                    responseData = await GenericFunctions_1.microsoftApiRequest.call(this, 'GET', `/v1.0/teams/${teamId}/channels/${channelId}`);
                }
                if (operation === 'getAll') {
                    const teamId = this.getNodeParameter('teamId', i);
                    const returnAll = this.getNodeParameter('returnAll', i);
                    if (returnAll) {
                        responseData = await GenericFunctions_1.microsoftApiRequestAllItems.call(this, 'value', 'GET', `/v1.0/teams/${teamId}/channels`);
                    }
                    else {
                        qs.limit = this.getNodeParameter('limit', i);
                        responseData = await GenericFunctions_1.microsoftApiRequestAllItems.call(this, 'value', 'GET', `/v1.0/teams/${teamId}/channels`, {});
                        responseData = responseData.splice(0, qs.limit);
                    }
                }
                if (operation === 'update') {
                    const teamId = this.getNodeParameter('teamId', i);
                    const channelId = this.getNodeParameter('channelId', i);
                    const updateFields = this.getNodeParameter('updateFields', i);
                    const body = {};
                    if (updateFields.name) {
                        body.displayName = updateFields.name;
                    }
                    if (updateFields.description) {
                        body.description = updateFields.description;
                    }
                    responseData = await GenericFunctions_1.microsoftApiRequest.call(this, 'PATCH', `/v1.0/teams/${teamId}/channels/${channelId}`, body);
                    responseData = { success: true };
                }
            }
            if (resource === 'channelMessage') {
                if (operation === 'create') {
                    const teamId = this.getNodeParameter('teamId', i);
                    const channelId = this.getNodeParameter('channelId', i);
                    const messageType = this.getNodeParameter('messageType', i);
                    const message = this.getNodeParameter('message', i);
                    const body = {
                        body: {
                            contentType: messageType,
                            content: message,
                        },
                    };
                    responseData = await GenericFunctions_1.microsoftApiRequest.call(this, 'POST', `/beta/teams/${teamId}/channels/${channelId}/messages`, body);
                }
                if (operation === 'getAll') {
                    const teamId = this.getNodeParameter('teamId', i);
                    const channelId = this.getNodeParameter('channelId', i);
                    const returnAll = this.getNodeParameter('returnAll', i);
                    if (returnAll) {
                        responseData = await GenericFunctions_1.microsoftApiRequestAllItems.call(this, 'value', 'GET', `/beta/teams/${teamId}/channels/${channelId}/messages`);
                    }
                    else {
                        qs.limit = this.getNodeParameter('limit', i);
                        responseData = await GenericFunctions_1.microsoftApiRequestAllItems.call(this, 'value', 'GET', `/beta/teams/${teamId}/channels/${channelId}/messages`, {});
                        responseData = responseData.splice(0, qs.limit);
                    }
                }
            }
            if (Array.isArray(responseData)) {
                returnData.push.apply(returnData, responseData);
            }
            else {
                returnData.push(responseData);
            }
        }
        return [this.helpers.returnJsonArray(returnData)];
    }
}
exports.MicrosoftTeams = MicrosoftTeams;
//# sourceMappingURL=MicrosoftTeams.node.js.map