"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MySql = void 0;
const mysql2 = require("mysql2/promise");
const GenericFunctions_1 = require("./GenericFunctions");
class MySql {
    constructor() {
        this.description = {
            displayName: 'MySQL',
            name: 'mySql',
            icon: 'file:mysql.png',
            group: ['input'],
            version: 1,
            description: 'Get, add and update data in MySQL.',
            defaults: {
                name: 'MySQL',
                color: '#4279a2',
            },
            inputs: ['main'],
            outputs: ['main'],
            credentials: [
                {
                    name: 'mySql',
                    required: true,
                },
            ],
            properties: [
                {
                    displayName: 'Operation',
                    name: 'operation',
                    type: 'options',
                    options: [
                        {
                            name: 'Execute Query',
                            value: 'executeQuery',
                            description: 'Execute an SQL query.',
                        },
                        {
                            name: 'Insert',
                            value: 'insert',
                            description: 'Insert rows in database.',
                        },
                        {
                            name: 'Update',
                            value: 'update',
                            description: 'Update rows in database.',
                        },
                    ],
                    default: 'insert',
                    description: 'The operation to perform.',
                },
                {
                    displayName: 'Query',
                    name: 'query',
                    type: 'string',
                    typeOptions: {
                        rows: 5,
                    },
                    displayOptions: {
                        show: {
                            operation: [
                                'executeQuery',
                            ],
                        },
                    },
                    default: '',
                    placeholder: 'SELECT id, name FROM product WHERE id < 40',
                    required: true,
                    description: 'The SQL query to execute.',
                },
                {
                    displayName: 'Table',
                    name: 'table',
                    type: 'string',
                    displayOptions: {
                        show: {
                            operation: [
                                'insert',
                            ],
                        },
                    },
                    default: '',
                    required: true,
                    description: 'Name of the table in which to insert data to.',
                },
                {
                    displayName: 'Columns',
                    name: 'columns',
                    type: 'string',
                    displayOptions: {
                        show: {
                            operation: [
                                'insert',
                            ],
                        },
                    },
                    default: '',
                    placeholder: 'id,name,description',
                    description: 'Comma separated list of the properties which should used as columns for the new rows.',
                },
                {
                    displayName: 'Table',
                    name: 'table',
                    type: 'string',
                    displayOptions: {
                        show: {
                            operation: [
                                'update',
                            ],
                        },
                    },
                    default: '',
                    required: true,
                    description: 'Name of the table in which to update data in',
                },
                {
                    displayName: 'Update Key',
                    name: 'updateKey',
                    type: 'string',
                    displayOptions: {
                        show: {
                            operation: [
                                'update',
                            ],
                        },
                    },
                    default: 'id',
                    required: true,
                    description: 'Name of the property which decides which rows in the database should be updated. Normally that would be "id".',
                },
                {
                    displayName: 'Columns',
                    name: 'columns',
                    type: 'string',
                    displayOptions: {
                        show: {
                            operation: [
                                'update',
                            ],
                        },
                    },
                    default: '',
                    placeholder: 'name,description',
                    description: 'Comma separated list of the properties which should used as columns for rows to update.',
                },
            ],
        };
    }
    async execute() {
        const credentials = this.getCredentials('mySql');
        if (credentials === undefined) {
            throw new Error('No credentials got returned!');
        }
        const connection = await mysql2.createConnection(credentials);
        const items = this.getInputData();
        const operation = this.getNodeParameter('operation', 0);
        let returnItems = [];
        if (operation === 'executeQuery') {
            const queryQueue = items.map((item, index) => {
                const rawQuery = this.getNodeParameter('query', index);
                return connection.query(rawQuery);
            });
            let queryResult = await Promise.all(queryQueue);
            queryResult = queryResult.reduce((collection, result) => {
                const [rows, fields] = result;
                if (Array.isArray(rows)) {
                    return collection.concat(rows);
                }
                collection.push(rows);
                return collection;
            }, []);
            returnItems = this.helpers.returnJsonArray(queryResult);
        }
        else if (operation === 'insert') {
            const table = this.getNodeParameter('table', 0);
            const columnString = this.getNodeParameter('columns', 0);
            const columns = columnString.split(',').map(column => column.trim());
            const insertItems = GenericFunctions_1.copyInputItems(items, columns);
            const insertPlaceholder = `(${columns.map(column => '?').join(',')})`;
            const insertSQL = `INSERT INTO ${table}(${columnString}) VALUES ${items.map(item => insertPlaceholder).join(',')};`;
            const queryItems = insertItems.reduce((collection, item) => collection.concat(Object.values(item)), []);
            const queryResult = await connection.query(insertSQL, queryItems);
            returnItems = this.helpers.returnJsonArray(queryResult[0]);
        }
        else if (operation === 'update') {
            const table = this.getNodeParameter('table', 0);
            const updateKey = this.getNodeParameter('updateKey', 0);
            const columnString = this.getNodeParameter('columns', 0);
            const columns = columnString.split(',').map(column => column.trim());
            if (!columns.includes(updateKey)) {
                columns.unshift(updateKey);
            }
            const updateItems = GenericFunctions_1.copyInputItems(items, columns);
            const updateSQL = `UPDATE ${table} SET ${columns.map(column => `${column} = ?`).join(',')} WHERE ${updateKey} = ?;`;
            const queryQueue = updateItems.map((item) => connection.query(updateSQL, Object.values(item).concat(item[updateKey])));
            let queryResult = await Promise.all(queryQueue);
            queryResult = queryResult.map(result => result[0]);
            returnItems = this.helpers.returnJsonArray(queryResult);
        }
        else {
            await connection.end();
            throw new Error(`The operation "${operation}" is not supported!`);
        }
        await connection.end();
        return this.prepareOutputData(returnItems);
    }
}
exports.MySql = MySql;
//# sourceMappingURL=MySql.node.js.map