"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NextCloud = void 0;
const n8n_core_1 = require("n8n-core");
const xml2js_1 = require("xml2js");
const GenericFunctions_1 = require("./GenericFunctions");
class NextCloud {
    constructor() {
        this.description = {
            displayName: 'Nextcloud',
            name: 'nextCloud',
            icon: 'file:nextcloud.png',
            group: ['input'],
            version: 1,
            subtitle: '={{$parameter["operation"] + ": " + $parameter["resource"]}}',
            description: 'Access data on Nextcloud',
            defaults: {
                name: 'Nextcloud',
                color: '#1cafff',
            },
            inputs: ['main'],
            outputs: ['main'],
            credentials: [
                {
                    name: 'nextCloudApi',
                    required: true,
                    displayOptions: {
                        show: {
                            authentication: [
                                'accessToken',
                            ],
                        },
                    },
                },
                {
                    name: 'nextCloudOAuth2Api',
                    required: true,
                    displayOptions: {
                        show: {
                            authentication: [
                                'oAuth2',
                            ],
                        },
                    },
                },
            ],
            properties: [
                {
                    displayName: 'Authentication',
                    name: 'authentication',
                    type: 'options',
                    options: [
                        {
                            name: 'Access Token',
                            value: 'accessToken',
                        },
                        {
                            name: 'OAuth2',
                            value: 'oAuth2',
                        },
                    ],
                    default: 'accessToken',
                    description: 'The resource to operate on.',
                },
                {
                    displayName: 'Resource',
                    name: 'resource',
                    type: 'options',
                    options: [
                        {
                            name: 'File',
                            value: 'file',
                        },
                        {
                            name: 'Folder',
                            value: 'folder',
                        },
                    ],
                    default: 'file',
                    description: 'The resource to operate on.',
                },
                {
                    displayName: 'Operation',
                    name: 'operation',
                    type: 'options',
                    displayOptions: {
                        show: {
                            resource: [
                                'file',
                            ],
                        },
                    },
                    options: [
                        {
                            name: 'Copy',
                            value: 'copy',
                            description: 'Copy a file',
                        },
                        {
                            name: 'Delete',
                            value: 'delete',
                            description: 'Delete a file',
                        },
                        {
                            name: 'Download',
                            value: 'download',
                            description: 'Download a file',
                        },
                        {
                            name: 'Move',
                            value: 'move',
                            description: 'Move a file',
                        },
                        {
                            name: 'Upload',
                            value: 'upload',
                            description: 'Upload a file',
                        },
                    ],
                    default: 'upload',
                    description: 'The operation to perform.',
                },
                {
                    displayName: 'Operation',
                    name: 'operation',
                    type: 'options',
                    displayOptions: {
                        show: {
                            resource: [
                                'folder',
                            ],
                        },
                    },
                    options: [
                        {
                            name: 'Copy',
                            value: 'copy',
                            description: 'Copy a folder',
                        },
                        {
                            name: 'Create',
                            value: 'create',
                            description: 'Create a folder',
                        },
                        {
                            name: 'Delete',
                            value: 'delete',
                            description: 'Delete a folder',
                        },
                        {
                            name: 'List',
                            value: 'list',
                            description: 'Return the contents of a given folder',
                        },
                        {
                            name: 'Move',
                            value: 'move',
                            description: 'Move a folder',
                        },
                    ],
                    default: 'create',
                    description: 'The operation to perform.',
                },
                {
                    displayName: 'From Path',
                    name: 'path',
                    type: 'string',
                    default: '',
                    required: true,
                    displayOptions: {
                        show: {
                            operation: [
                                'copy',
                            ],
                            resource: [
                                'file',
                                'folder',
                            ],
                        },
                    },
                    placeholder: '/invoices/original.txt',
                    description: 'The path of file or folder to copy.',
                },
                {
                    displayName: 'To Path',
                    name: 'toPath',
                    type: 'string',
                    default: '',
                    required: true,
                    displayOptions: {
                        show: {
                            operation: [
                                'copy',
                            ],
                            resource: [
                                'file',
                                'folder',
                            ],
                        },
                    },
                    placeholder: '/invoices/copy.txt',
                    description: 'The destination path of file or folder.',
                },
                {
                    displayName: 'Delete Path',
                    name: 'path',
                    type: 'string',
                    default: '',
                    required: true,
                    displayOptions: {
                        show: {
                            operation: [
                                'delete',
                            ],
                            resource: [
                                'file',
                                'folder',
                            ],
                        },
                    },
                    placeholder: 'invoices/2019/invoice_1.pdf',
                    description: 'The path to delete. Can be a single file or a whole folder.',
                },
                {
                    displayName: 'From Path',
                    name: 'path',
                    type: 'string',
                    default: '',
                    required: true,
                    displayOptions: {
                        show: {
                            operation: [
                                'move',
                            ],
                            resource: [
                                'file',
                                'folder',
                            ],
                        },
                    },
                    placeholder: '/invoices/old_name.txt',
                    description: 'The path of file or folder to move.',
                },
                {
                    displayName: 'To Path',
                    name: 'toPath',
                    type: 'string',
                    default: '',
                    required: true,
                    displayOptions: {
                        show: {
                            operation: [
                                'move',
                            ],
                            resource: [
                                'file',
                                'folder',
                            ],
                        },
                    },
                    placeholder: '/invoices/new_name.txt',
                    description: 'The new path of file or folder.',
                },
                {
                    displayName: 'File Path',
                    name: 'path',
                    type: 'string',
                    default: '',
                    required: true,
                    displayOptions: {
                        show: {
                            operation: [
                                'download',
                            ],
                            resource: [
                                'file',
                            ],
                        },
                    },
                    placeholder: 'invoices/2019/invoice_1.pdf',
                    description: 'The file path of the file to download. Has to contain the full path.',
                },
                {
                    displayName: 'Binary Property',
                    name: 'binaryPropertyName',
                    type: 'string',
                    default: 'data',
                    required: true,
                    displayOptions: {
                        show: {
                            operation: [
                                'download',
                            ],
                            resource: [
                                'file',
                            ],
                        },
                    },
                    description: 'Name of the binary property to which to<br />write the data of the read file.',
                },
                {
                    displayName: 'File Path',
                    name: 'path',
                    type: 'string',
                    default: '',
                    required: true,
                    displayOptions: {
                        show: {
                            operation: [
                                'upload',
                            ],
                            resource: [
                                'file',
                            ],
                        },
                    },
                    placeholder: 'invoices/2019/invoice_1.pdf',
                    description: 'The file path of the file to upload. Has to contain the full path. The parent folder has to exist. Existing files get overwritten.',
                },
                {
                    displayName: 'Binary Data',
                    name: 'binaryDataUpload',
                    type: 'boolean',
                    default: false,
                    required: true,
                    displayOptions: {
                        show: {
                            operation: [
                                'upload',
                            ],
                            resource: [
                                'file',
                            ],
                        },
                    },
                    description: '',
                },
                {
                    displayName: 'File Content',
                    name: 'fileContent',
                    type: 'string',
                    default: '',
                    displayOptions: {
                        show: {
                            binaryDataUpload: [
                                false,
                            ],
                            operation: [
                                'upload',
                            ],
                            resource: [
                                'file',
                            ],
                        },
                    },
                    placeholder: '',
                    description: 'The text content of the file to upload.',
                },
                {
                    displayName: 'Binary Property',
                    name: 'binaryPropertyName',
                    type: 'string',
                    default: 'data',
                    required: true,
                    displayOptions: {
                        show: {
                            binaryDataUpload: [
                                true,
                            ],
                            operation: [
                                'upload',
                            ],
                            resource: [
                                'file',
                            ],
                        },
                    },
                    placeholder: '',
                    description: 'Name of the binary property which contains<br />the data for the file to be uploaded.',
                },
                {
                    displayName: 'Folder',
                    name: 'path',
                    type: 'string',
                    default: '',
                    required: true,
                    displayOptions: {
                        show: {
                            operation: [
                                'create',
                            ],
                            resource: [
                                'folder',
                            ],
                        },
                    },
                    placeholder: 'invoices/2019',
                    description: 'The folder to create. The parent folder has to exist.',
                },
                {
                    displayName: 'Folder Path',
                    name: 'path',
                    type: 'string',
                    default: '',
                    displayOptions: {
                        show: {
                            operation: [
                                'list',
                            ],
                            resource: [
                                'folder',
                            ],
                        },
                    },
                    placeholder: 'invoices/2019/',
                    description: 'The path of which to list the content.',
                },
            ],
        };
    }
    async execute() {
        const items = this.getInputData().slice();
        const returnData = [];
        const authenticationMethod = this.getNodeParameter('authentication', 0);
        let credentials;
        if (authenticationMethod === 'accessToken') {
            credentials = this.getCredentials('nextCloudApi');
        }
        else {
            credentials = this.getCredentials('nextCloudOAuth2Api');
        }
        if (credentials === undefined) {
            throw new Error('No credentials got returned!');
        }
        const resource = this.getNodeParameter('resource', 0);
        const operation = this.getNodeParameter('operation', 0);
        let endpoint = '';
        let requestMethod = '';
        let responseData;
        let body = '';
        const headers = {};
        for (let i = 0; i < items.length; i++) {
            if (resource === 'file') {
                if (operation === 'download') {
                    requestMethod = 'GET';
                    endpoint = this.getNodeParameter('path', i);
                }
                else if (operation === 'upload') {
                    requestMethod = 'PUT';
                    endpoint = this.getNodeParameter('path', i);
                    if (this.getNodeParameter('binaryDataUpload', i) === true) {
                        const item = items[i];
                        if (item.binary === undefined) {
                            throw new Error('No binary data exists on item!');
                        }
                        const propertyNameUpload = this.getNodeParameter('binaryPropertyName', i);
                        if (item.binary[propertyNameUpload] === undefined) {
                            throw new Error(`No binary data property "${propertyNameUpload}" does not exists on item!`);
                        }
                        body = Buffer.from(item.binary[propertyNameUpload].data, n8n_core_1.BINARY_ENCODING);
                    }
                    else {
                        body = this.getNodeParameter('fileContent', i);
                    }
                }
            }
            else if (resource === 'folder') {
                if (operation === 'create') {
                    requestMethod = 'MKCOL';
                    endpoint = this.getNodeParameter('path', i);
                }
                else if (operation === 'list') {
                    requestMethod = 'PROPFIND';
                    endpoint = this.getNodeParameter('path', i);
                }
            }
            if (['file', 'folder'].includes(resource)) {
                if (operation === 'copy') {
                    requestMethod = 'COPY';
                    endpoint = this.getNodeParameter('path', i);
                    const toPath = this.getNodeParameter('toPath', i);
                    headers.Destination = `${credentials.webDavUrl}/${encodeURI(toPath)}`;
                }
                else if (operation === 'delete') {
                    requestMethod = 'DELETE';
                    endpoint = this.getNodeParameter('path', i);
                }
                else if (operation === 'move') {
                    requestMethod = 'MOVE';
                    endpoint = this.getNodeParameter('path', i);
                    const toPath = this.getNodeParameter('toPath', i);
                    headers.Destination = `${credentials.webDavUrl}/${encodeURI(toPath)}`;
                }
            }
            else {
                throw new Error(`The resource "${resource}" is not known!`);
            }
            let webDavUrl = credentials.webDavUrl;
            if (webDavUrl.slice(-1) === '/') {
                webDavUrl = webDavUrl.slice(0, -1);
            }
            let encoding = undefined;
            if (resource === 'file' && operation === 'download') {
                encoding = null;
            }
            try {
                responseData = await GenericFunctions_1.nextCloudApiRequest.call(this, requestMethod, endpoint, body, headers, encoding);
            }
            catch (error) {
                if (this.continueOnFail() === true) {
                    returnData.push({ error });
                    continue;
                }
                throw error;
            }
            if (resource === 'file' && operation === 'download') {
                const newItem = {
                    json: items[i].json,
                    binary: {},
                };
                if (items[i].binary !== undefined) {
                    Object.assign(newItem.binary, items[i].binary);
                }
                items[i] = newItem;
                const binaryPropertyName = this.getNodeParameter('binaryPropertyName', i);
                items[i].binary[binaryPropertyName] = await this.helpers.prepareBinaryData(responseData, endpoint);
            }
            else if (resource === 'folder' && operation === 'list') {
                const jsonResponseData = await new Promise((resolve, reject) => {
                    xml2js_1.parseString(responseData, { explicitArray: false }, (err, data) => {
                        if (err) {
                            return reject(err);
                        }
                        resolve(data);
                    });
                });
                const propNames = {
                    'd:getlastmodified': 'lastModified',
                    'd:getcontentlength': 'contentLength',
                    'd:getcontenttype': 'contentType',
                };
                if (jsonResponseData['d:multistatus'] !== undefined &&
                    jsonResponseData['d:multistatus'] !== null &&
                    jsonResponseData['d:multistatus']['d:response'] !== undefined &&
                    jsonResponseData['d:multistatus']['d:response'] !== null) {
                    let skippedFirst = false;
                    for (const item of jsonResponseData['d:multistatus']['d:response']) {
                        if (skippedFirst === false) {
                            skippedFirst = true;
                            continue;
                        }
                        const newItem = {};
                        newItem.path = item['d:href'].slice(19);
                        const props = item['d:propstat'][0]['d:prop'];
                        for (const propName of Object.keys(propNames)) {
                            if (props[propName] !== undefined) {
                                newItem[propNames[propName]] = props[propName];
                            }
                        }
                        if (props['d:resourcetype'] === '') {
                            newItem.type = 'file';
                        }
                        else {
                            newItem.type = 'folder';
                        }
                        newItem.eTag = props['d:getetag'].slice(1, -1);
                        returnData.push(newItem);
                    }
                }
            }
            else {
                returnData.push(responseData);
            }
        }
        if (resource === 'file' && operation === 'download') {
            return this.prepareOutputData(items);
        }
        else {
            return [this.helpers.returnJsonArray(returnData)];
        }
    }
}
exports.NextCloud = NextCloud;
//# sourceMappingURL=NextCloud.node.js.map