"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Pipedrive = void 0;
const n8n_core_1 = require("n8n-core");
const GenericFunctions_1 = require("./GenericFunctions");
function addAdditionalFields(body, additionalFields) {
    for (const key of Object.keys(additionalFields)) {
        if (key === 'customProperties' && additionalFields.customProperties.property !== undefined) {
            for (const customProperty of additionalFields.customProperties.property) {
                body[customProperty.name] = customProperty.value;
            }
        }
        else {
            body[key] = additionalFields[key];
        }
    }
}
class Pipedrive {
    constructor() {
        this.description = {
            displayName: 'Pipedrive',
            name: 'pipedrive',
            icon: 'file:pipedrive.png',
            group: ['transform'],
            version: 1,
            subtitle: '={{$parameter["operation"] + ": " + $parameter["resource"]}}',
            description: 'Create and edit data in Pipedrive',
            defaults: {
                name: 'Pipedrive',
                color: '#227722',
            },
            inputs: ['main'],
            outputs: ['main'],
            credentials: [
                {
                    name: 'pipedriveApi',
                    required: true,
                    displayOptions: {
                        show: {
                            authentication: [
                                'apiToken',
                            ],
                        },
                    },
                },
                {
                    name: 'pipedriveOAuth2Api',
                    required: true,
                    displayOptions: {
                        show: {
                            authentication: [
                                'oAuth2',
                            ],
                        },
                    },
                },
            ],
            properties: [
                {
                    displayName: 'Authentication',
                    name: 'authentication',
                    type: 'options',
                    options: [
                        {
                            name: 'API Token',
                            value: 'apiToken',
                        },
                        {
                            name: 'OAuth2',
                            value: 'oAuth2',
                        },
                    ],
                    default: 'apiToken',
                    description: 'Method of authentication.',
                },
                {
                    displayName: 'Resource',
                    name: 'resource',
                    type: 'options',
                    options: [
                        {
                            name: 'Activity',
                            value: 'activity',
                        },
                        {
                            name: 'Deal',
                            value: 'deal',
                        },
                        {
                            name: 'File',
                            value: 'file',
                        },
                        {
                            name: 'Note',
                            value: 'note',
                        },
                        {
                            name: 'Organization',
                            value: 'organization',
                        },
                        {
                            name: 'Person',
                            value: 'person',
                        },
                        {
                            name: 'Product',
                            value: 'product',
                        },
                    ],
                    default: 'deal',
                    description: 'The resource to operate on.',
                },
                {
                    displayName: 'Operation',
                    name: 'operation',
                    type: 'options',
                    displayOptions: {
                        show: {
                            resource: [
                                'activity',
                            ],
                        },
                    },
                    options: [
                        {
                            name: 'Create',
                            value: 'create',
                            description: 'Create an activity',
                        },
                        {
                            name: 'Delete',
                            value: 'delete',
                            description: 'Delete an activity',
                        },
                        {
                            name: 'Get',
                            value: 'get',
                            description: 'Get data of an activity',
                        },
                        {
                            name: 'Get All',
                            value: 'getAll',
                            description: 'Get data of all activities',
                        },
                        {
                            name: 'Update',
                            value: 'update',
                            description: 'Update an activity',
                        },
                    ],
                    default: 'create',
                    description: 'The operation to perform.',
                },
                {
                    displayName: 'Operation',
                    name: 'operation',
                    type: 'options',
                    displayOptions: {
                        show: {
                            resource: [
                                'deal',
                            ],
                        },
                    },
                    options: [
                        {
                            name: 'Create',
                            value: 'create',
                            description: 'Create a deal',
                        },
                        {
                            name: 'Delete',
                            value: 'delete',
                            description: 'Delete a deal',
                        },
                        {
                            name: 'Duplicate',
                            value: 'duplicate',
                            description: 'Duplicate a deal',
                        },
                        {
                            name: 'Get',
                            value: 'get',
                            description: 'Get data of a deal',
                        },
                        {
                            name: 'Get All',
                            value: 'getAll',
                            description: 'Get data of all deals',
                        },
                        {
                            name: 'Update',
                            value: 'update',
                            description: 'Update a deal',
                        },
                    ],
                    default: 'create',
                    description: 'The operation to perform.',
                },
                {
                    displayName: 'Operation',
                    name: 'operation',
                    type: 'options',
                    displayOptions: {
                        show: {
                            resource: [
                                'file',
                            ],
                        },
                    },
                    options: [
                        {
                            name: 'Create',
                            value: 'create',
                            description: 'Create a file',
                        },
                        {
                            name: 'Delete',
                            value: 'delete',
                            description: 'Delete a file',
                        },
                        {
                            name: 'Download',
                            value: 'download',
                            description: 'Download a file',
                        },
                        {
                            name: 'Get',
                            value: 'get',
                            description: 'Get data of a file',
                        },
                    ],
                    default: 'create',
                    description: 'The operation to perform.',
                },
                {
                    displayName: 'Operation',
                    name: 'operation',
                    type: 'options',
                    displayOptions: {
                        show: {
                            resource: [
                                'note',
                            ],
                        },
                    },
                    options: [
                        {
                            name: 'Create',
                            value: 'create',
                            description: 'Create a note',
                        },
                        {
                            name: 'Delete',
                            value: 'delete',
                            description: 'Delete a note',
                        },
                        {
                            name: 'Get',
                            value: 'get',
                            description: 'Get data of a note',
                        },
                        {
                            name: 'Get All',
                            value: 'getAll',
                            description: 'Get data of all notes',
                        },
                        {
                            name: 'Update',
                            value: 'update',
                            description: 'Update a note',
                        },
                    ],
                    default: 'create',
                    description: 'The operation to perform.',
                },
                {
                    displayName: 'Operation',
                    name: 'operation',
                    type: 'options',
                    displayOptions: {
                        show: {
                            resource: [
                                'organization',
                            ],
                        },
                    },
                    options: [
                        {
                            name: 'Create',
                            value: 'create',
                            description: 'Create an organization',
                        },
                        {
                            name: 'Delete',
                            value: 'delete',
                            description: 'Delete an organization',
                        },
                        {
                            name: 'Get',
                            value: 'get',
                            description: 'Get data of an organization',
                        },
                        {
                            name: 'Get All',
                            value: 'getAll',
                            description: 'Get data of all organizations',
                        },
                    ],
                    default: 'create',
                    description: 'The operation to perform.',
                },
                {
                    displayName: 'Operation',
                    name: 'operation',
                    type: 'options',
                    displayOptions: {
                        show: {
                            resource: [
                                'person',
                            ],
                        },
                    },
                    options: [
                        {
                            name: 'Create',
                            value: 'create',
                            description: 'Create a person',
                        },
                        {
                            name: 'Delete',
                            value: 'delete',
                            description: 'Delete a person',
                        },
                        {
                            name: 'Get',
                            value: 'get',
                            description: 'Get data of a person',
                        },
                        {
                            name: 'Get All',
                            value: 'getAll',
                            description: 'Get data of all persons',
                        },
                        {
                            name: 'Search',
                            value: 'search',
                            description: 'Search all persons',
                        },
                        {
                            name: 'Update',
                            value: 'update',
                            description: 'Update a person',
                        },
                    ],
                    default: 'create',
                    description: 'The operation to perform.',
                },
                {
                    displayName: 'Operation',
                    name: 'operation',
                    type: 'options',
                    displayOptions: {
                        show: {
                            resource: [
                                'product',
                            ],
                        },
                    },
                    options: [
                        {
                            name: 'Get All',
                            value: 'getAll',
                            description: 'Get data of all products',
                        },
                    ],
                    default: 'getAll',
                    description: 'The operation to perform.',
                },
                {
                    displayName: 'Subject',
                    name: 'subject',
                    type: 'string',
                    default: '',
                    required: true,
                    displayOptions: {
                        show: {
                            operation: [
                                'create',
                            ],
                            resource: [
                                'activity',
                            ],
                        },
                    },
                    description: 'The subject of the activity to create',
                },
                {
                    displayName: 'Done',
                    name: 'done',
                    type: 'options',
                    displayOptions: {
                        show: {
                            operation: [
                                'create',
                            ],
                            resource: [
                                'activity',
                            ],
                        },
                    },
                    options: [
                        {
                            name: 'Not done',
                            value: '0',
                        },
                        {
                            name: 'Done',
                            value: '1',
                        },
                    ],
                    default: '0',
                    description: 'Whether the activity is done or not.',
                },
                {
                    displayName: 'Type',
                    name: 'type',
                    type: 'string',
                    default: '',
                    required: true,
                    displayOptions: {
                        show: {
                            operation: [
                                'create',
                            ],
                            resource: [
                                'activity',
                            ],
                        },
                    },
                    placeholder: 'call',
                    description: 'Type of the activity like "call", "meeting", ...',
                },
                {
                    displayName: 'Additional Fields',
                    name: 'additionalFields',
                    type: 'collection',
                    placeholder: 'Add Field',
                    displayOptions: {
                        show: {
                            operation: [
                                'create',
                            ],
                            resource: [
                                'activity',
                            ],
                        },
                    },
                    default: {},
                    options: [
                        {
                            displayName: 'Deal ID',
                            name: 'deal_id',
                            type: 'number',
                            default: 0,
                            description: 'ID of the deal this activity will be associated with',
                        },
                        {
                            displayName: 'Note',
                            name: 'note',
                            type: 'string',
                            typeOptions: {
                                alwaysOpenEditWindow: true,
                                rows: 5,
                            },
                            default: '',
                            description: 'Note of the activity (HTML format)',
                        },
                        {
                            displayName: 'Organization ID',
                            name: 'org_id',
                            type: 'number',
                            default: 0,
                            description: 'ID of the organization this activity will be associated with',
                        },
                        {
                            displayName: 'Person ID',
                            name: 'person_id',
                            type: 'number',
                            default: 0,
                            description: 'ID of the person this activity will be associated with',
                        },
                        {
                            displayName: 'User ID',
                            name: 'user_id',
                            type: 'number',
                            default: 0,
                            description: 'ID of the user whom the activity will be assigned to. If omitted, the activity will be assigned to the authorized user.',
                        },
                        {
                            displayName: 'Custom Properties',
                            name: 'customProperties',
                            placeholder: 'Add Custom Property',
                            description: 'Adds a custom property to set also values which have not been predefined.',
                            type: 'fixedCollection',
                            typeOptions: {
                                multipleValues: true,
                            },
                            default: {},
                            options: [
                                {
                                    name: 'property',
                                    displayName: 'Property',
                                    values: [
                                        {
                                            displayName: 'Property Name',
                                            name: 'name',
                                            type: 'string',
                                            default: '',
                                            description: 'Name of the property to set.',
                                        },
                                        {
                                            displayName: 'Property Value',
                                            name: 'value',
                                            type: 'string',
                                            default: '',
                                            description: 'Value of the property to set.',
                                        },
                                    ],
                                },
                            ],
                        },
                    ],
                },
                {
                    displayName: 'Activity ID',
                    name: 'activityId',
                    type: 'number',
                    displayOptions: {
                        show: {
                            operation: [
                                'delete',
                            ],
                            resource: [
                                'activity',
                            ],
                        },
                    },
                    default: 0,
                    required: true,
                    description: 'ID of the activity to delete.',
                },
                {
                    displayName: 'Activity ID',
                    name: 'activityId',
                    type: 'number',
                    displayOptions: {
                        show: {
                            operation: [
                                'get',
                            ],
                            resource: [
                                'activity',
                            ],
                        },
                    },
                    default: 0,
                    required: true,
                    description: 'ID of the activity to get.',
                },
                {
                    displayName: 'Activity ID',
                    name: 'activityId',
                    type: 'number',
                    displayOptions: {
                        show: {
                            operation: [
                                'update',
                            ],
                            resource: [
                                'activity',
                            ],
                        },
                    },
                    default: 0,
                    required: true,
                    description: 'ID of the activity to update.',
                },
                {
                    displayName: 'Update Fields',
                    name: 'updateFields',
                    type: 'collection',
                    placeholder: 'Add Field',
                    displayOptions: {
                        show: {
                            operation: [
                                'update',
                            ],
                            resource: [
                                'activity',
                            ],
                        },
                    },
                    default: {},
                    options: [
                        {
                            displayName: 'Deal ID',
                            name: 'deal_id',
                            type: 'number',
                            default: 0,
                            description: 'ID of the deal this activity will be associated with',
                        },
                        {
                            displayName: 'Done',
                            name: 'done',
                            type: 'options',
                            options: [
                                {
                                    name: 'Not done',
                                    value: '0',
                                },
                                {
                                    name: 'Done',
                                    value: '1',
                                },
                            ],
                            default: '0',
                            description: 'Whether the activity is done or not.',
                        },
                        {
                            displayName: 'Note',
                            name: 'note',
                            type: 'string',
                            typeOptions: {
                                alwaysOpenEditWindow: true,
                                rows: 5,
                            },
                            default: '',
                            description: 'Note of the activity (HTML format)',
                        },
                        {
                            displayName: 'Organization ID',
                            name: 'org_id',
                            type: 'number',
                            default: 0,
                            description: 'ID of the organization this activity will be associated with',
                        },
                        {
                            displayName: 'Person ID',
                            name: 'person_id',
                            type: 'number',
                            default: 0,
                            description: 'ID of the person this activity will be associated with',
                        },
                        {
                            displayName: 'Subject',
                            name: 'subject',
                            type: 'string',
                            default: '',
                            description: 'The subject of the activity',
                        },
                        {
                            displayName: 'Type',
                            name: 'type',
                            type: 'string',
                            default: '',
                            placeholder: 'call',
                            description: 'Type of the activity like "call", "meeting", ...',
                        },
                        {
                            displayName: 'User ID',
                            name: 'user_id',
                            type: 'number',
                            default: 0,
                            description: 'ID of the user whom the activity will be assigned to. If omitted, the activity will be assigned to the authorized user.',
                        },
                        {
                            displayName: 'Custom Properties',
                            name: 'customProperties',
                            placeholder: 'Add Custom Property',
                            description: 'Adds a custom property to set also values which have not been predefined.',
                            type: 'fixedCollection',
                            typeOptions: {
                                multipleValues: true,
                            },
                            default: {},
                            options: [
                                {
                                    name: 'property',
                                    displayName: 'Property',
                                    values: [
                                        {
                                            displayName: 'Property Name',
                                            name: 'name',
                                            type: 'string',
                                            default: '',
                                            description: 'Name of the property to set.',
                                        },
                                        {
                                            displayName: 'Property Value',
                                            name: 'value',
                                            type: 'string',
                                            default: '',
                                            description: 'Value of the property to set.',
                                        },
                                    ],
                                },
                            ],
                        },
                    ],
                },
                {
                    displayName: 'Title',
                    name: 'title',
                    type: 'string',
                    default: '',
                    required: true,
                    displayOptions: {
                        show: {
                            operation: [
                                'create',
                            ],
                            resource: [
                                'deal',
                            ],
                        },
                    },
                    description: 'The title of the deal to create',
                },
                {
                    displayName: 'Additional Fields',
                    name: 'additionalFields',
                    type: 'collection',
                    placeholder: 'Add Field',
                    displayOptions: {
                        show: {
                            operation: [
                                'create',
                            ],
                            resource: [
                                'deal',
                            ],
                        },
                    },
                    default: {},
                    options: [
                        {
                            displayName: 'Currency',
                            name: 'currency',
                            type: 'string',
                            default: 'USD',
                            description: 'Currency of the deal. Accepts a 3-character currency code. Like EUR, USD, ...',
                        },
                        {
                            displayName: 'Lost Reason',
                            name: 'lost_reason',
                            type: 'string',
                            default: '',
                            description: 'Reason why the deal was lost.',
                        },
                        {
                            displayName: 'Organization ID',
                            name: 'org_id',
                            type: 'number',
                            default: 0,
                            description: 'ID of the organization this deal will be associated with.',
                        },
                        {
                            displayName: 'Person ID',
                            name: 'person_id',
                            type: 'number',
                            default: 0,
                            description: 'ID of the person this deal will be associated with.',
                        },
                        {
                            displayName: 'Probability',
                            name: 'probability',
                            type: 'number',
                            typeOptions: {
                                minValue: 0,
                                maxValue: 100,
                            },
                            default: 0,
                            description: 'Deal success probability percentage.',
                        },
                        {
                            displayName: 'Stage ID',
                            name: 'stage_id',
                            type: 'number',
                            default: 0,
                            description: 'ID of the stage this deal will be placed in a pipeline. If omitted, the deal will be placed in the first stage of the default pipeline.',
                        },
                        {
                            displayName: 'Status',
                            name: 'status',
                            type: 'options',
                            options: [
                                {
                                    name: 'Open',
                                    value: 'open',
                                },
                                {
                                    name: 'Won',
                                    value: 'won',
                                },
                                {
                                    name: 'Lost',
                                    value: 'lost',
                                },
                                {
                                    name: 'Deleted',
                                    value: 'deleted',
                                },
                            ],
                            default: 'open',
                            description: 'The status of the deal. If not provided it will automatically be set to "open".',
                        },
                        {
                            displayName: 'User ID',
                            name: 'user_id',
                            type: 'number',
                            default: 0,
                            description: 'ID of the user who will be marked as the owner of this deal. If omitted, the authorized user ID will be used.',
                        },
                        {
                            displayName: 'Value',
                            name: 'value',
                            type: 'number',
                            default: 0,
                            description: 'Value of the deal. If not set it will automatically be set to 0.',
                        },
                        {
                            displayName: 'Visible to',
                            name: 'visible_to',
                            type: 'options',
                            options: [
                                {
                                    name: 'Owner & followers (private)',
                                    value: '1',
                                },
                                {
                                    name: 'Entire company (shared)',
                                    value: '3',
                                },
                            ],
                            default: '3',
                            description: 'Visibility of the deal. If omitted, visibility will be set to the default visibility setting of this item type for the authorized user.',
                        },
                        {
                            displayName: 'Custom Properties',
                            name: 'customProperties',
                            placeholder: 'Add Custom Property',
                            description: 'Adds a custom property to set also values which have not been predefined.',
                            type: 'fixedCollection',
                            typeOptions: {
                                multipleValues: true,
                            },
                            default: {},
                            options: [
                                {
                                    name: 'property',
                                    displayName: 'Property',
                                    values: [
                                        {
                                            displayName: 'Property Name',
                                            name: 'name',
                                            type: 'string',
                                            default: '',
                                            description: 'Name of the property to set.',
                                        },
                                        {
                                            displayName: 'Property Value',
                                            name: 'value',
                                            type: 'string',
                                            default: '',
                                            description: 'Value of the property to set.',
                                        },
                                    ],
                                },
                            ],
                        },
                    ],
                },
                {
                    displayName: 'Deal ID',
                    name: 'dealId',
                    type: 'number',
                    displayOptions: {
                        show: {
                            operation: [
                                'delete',
                            ],
                            resource: [
                                'deal',
                            ],
                        },
                    },
                    default: 0,
                    required: true,
                    description: 'ID of the deal to delete.',
                },
                {
                    displayName: 'Deal ID',
                    name: 'dealId',
                    type: 'number',
                    displayOptions: {
                        show: {
                            operation: [
                                'duplicate',
                            ],
                            resource: [
                                'deal',
                            ],
                        },
                    },
                    default: 0,
                    required: true,
                    description: 'ID of the deal to duplicate.',
                },
                {
                    displayName: 'Deal ID',
                    name: 'dealId',
                    type: 'number',
                    displayOptions: {
                        show: {
                            operation: [
                                'get',
                            ],
                            resource: [
                                'deal',
                            ],
                        },
                    },
                    default: 0,
                    required: true,
                    description: 'ID of the deal to get.',
                },
                {
                    displayName: 'Deal ID',
                    name: 'dealId',
                    type: 'number',
                    displayOptions: {
                        show: {
                            operation: [
                                'update',
                            ],
                            resource: [
                                'deal',
                            ],
                        },
                    },
                    default: 0,
                    required: true,
                    description: 'ID of the deal to update.',
                },
                {
                    displayName: 'Update Fields',
                    name: 'updateFields',
                    type: 'collection',
                    placeholder: 'Add Field',
                    displayOptions: {
                        show: {
                            operation: [
                                'update',
                            ],
                            resource: [
                                'deal',
                            ],
                        },
                    },
                    default: {},
                    options: [
                        {
                            displayName: 'Currency',
                            name: 'currency',
                            type: 'string',
                            default: 'USD',
                            description: 'Currency of the deal. Accepts a 3-character currency code. Like EUR, USD, ...',
                        },
                        {
                            displayName: 'Lost Reason',
                            name: 'lost_reason',
                            type: 'string',
                            default: '',
                            description: 'Reason why the deal was lost.',
                        },
                        {
                            displayName: 'Organization ID',
                            name: 'org_id',
                            type: 'number',
                            default: 0,
                            description: 'ID of the organization this deal will be associated with.',
                        },
                        {
                            displayName: 'Person ID',
                            name: 'person_id',
                            type: 'number',
                            default: 0,
                            description: 'ID of the person this deal will be associated with.',
                        },
                        {
                            displayName: 'Probability',
                            name: 'probability',
                            type: 'number',
                            typeOptions: {
                                minValue: 0,
                                maxValue: 100,
                            },
                            default: 0,
                            description: 'Deal success probability percentage.',
                        },
                        {
                            displayName: 'Stage ID',
                            name: 'stage_id',
                            type: 'number',
                            default: 0,
                            description: 'ID of the stage this deal will be placed in a pipeline. If omitted, the deal will be placed in the first stage of the default pipeline.',
                        },
                        {
                            displayName: 'Status',
                            name: 'status',
                            type: 'options',
                            options: [
                                {
                                    name: 'Open',
                                    value: 'open',
                                },
                                {
                                    name: 'Won',
                                    value: 'won',
                                },
                                {
                                    name: 'Lost',
                                    value: 'lost',
                                },
                                {
                                    name: 'Deleted',
                                    value: 'deleted',
                                },
                            ],
                            default: 'open',
                            description: 'The status of the deal. If not provided it will automatically be set to "open".',
                        },
                        {
                            displayName: 'Title',
                            name: 'title',
                            type: 'string',
                            default: '',
                            description: 'The title of the deal',
                        },
                        {
                            displayName: 'Value',
                            name: 'value',
                            type: 'number',
                            default: 0,
                            description: 'Value of the deal. If not set it will automatically be set to 0.',
                        },
                        {
                            displayName: 'Visible to',
                            name: 'visible_to',
                            type: 'options',
                            options: [
                                {
                                    name: 'Owner & followers (private)',
                                    value: '1',
                                },
                                {
                                    name: 'Entire company (shared)',
                                    value: '3',
                                },
                            ],
                            default: '3',
                            description: 'Visibility of the deal. If omitted, visibility will be set to the default visibility setting of this item type for the authorized user.',
                        },
                        {
                            displayName: 'Custom Properties',
                            name: 'customProperties',
                            placeholder: 'Add Custom Property',
                            description: 'Adds a custom property to set also values which have not been predefined.',
                            type: 'fixedCollection',
                            typeOptions: {
                                multipleValues: true,
                            },
                            default: {},
                            options: [
                                {
                                    name: 'property',
                                    displayName: 'Property',
                                    values: [
                                        {
                                            displayName: 'Property Name',
                                            name: 'name',
                                            type: 'string',
                                            default: '',
                                            description: 'Name of the property to set.',
                                        },
                                        {
                                            displayName: 'Property Value',
                                            name: 'value',
                                            type: 'string',
                                            default: '',
                                            description: 'Value of the property to set.',
                                        },
                                    ],
                                },
                            ],
                        },
                    ],
                },
                {
                    displayName: 'Binary Property',
                    name: 'binaryPropertyName',
                    type: 'string',
                    default: 'data',
                    required: true,
                    displayOptions: {
                        show: {
                            operation: [
                                'create',
                            ],
                            resource: [
                                'file',
                            ],
                        },
                    },
                    placeholder: '',
                    description: 'Name of the binary property which contains<br />the data for the file to be created.',
                },
                {
                    displayName: 'Additional Fields',
                    name: 'additionalFields',
                    type: 'collection',
                    placeholder: 'Add Field',
                    displayOptions: {
                        show: {
                            operation: [
                                'create',
                            ],
                            resource: [
                                'file',
                            ],
                        },
                    },
                    default: {},
                    options: [
                        {
                            displayName: 'Activity ID',
                            name: 'activity_id',
                            type: 'number',
                            default: 0,
                            description: 'ID of the activite this file will be associated with.',
                        },
                        {
                            displayName: 'Deal ID',
                            name: 'deal_id',
                            type: 'number',
                            default: 0,
                            description: 'ID of the deal this file will be associated with',
                        },
                        {
                            displayName: 'Organization ID',
                            name: 'org_id',
                            type: 'number',
                            default: 0,
                            description: 'ID of the organization this file will be associated with.',
                        },
                        {
                            displayName: 'Person ID',
                            name: 'person_id',
                            type: 'number',
                            default: 0,
                            description: 'ID of the person this file will be associated with.',
                        },
                        {
                            displayName: 'Product ID',
                            name: 'product_id',
                            type: 'number',
                            default: 0,
                            description: 'ID of the person this file will be associated with.',
                        },
                    ],
                },
                {
                    displayName: 'File ID',
                    name: 'fileId',
                    type: 'number',
                    displayOptions: {
                        show: {
                            operation: [
                                'delete',
                            ],
                            resource: [
                                'file',
                            ],
                        },
                    },
                    default: 0,
                    required: true,
                    description: 'ID of the file to delete.',
                },
                {
                    displayName: 'File ID',
                    name: 'fileId',
                    type: 'number',
                    displayOptions: {
                        show: {
                            operation: [
                                'download',
                            ],
                            resource: [
                                'file',
                            ],
                        },
                    },
                    default: 0,
                    required: true,
                    description: 'ID of the file to download.',
                },
                {
                    displayName: 'Binary Property',
                    name: 'binaryPropertyName',
                    type: 'string',
                    required: true,
                    default: 'data',
                    displayOptions: {
                        show: {
                            operation: [
                                'download',
                            ],
                            resource: [
                                'file',
                            ],
                        },
                    },
                    description: 'Name of the binary property to which to<br />write the data of the downloaded file.',
                },
                {
                    displayName: 'File ID',
                    name: 'fileId',
                    type: 'number',
                    displayOptions: {
                        show: {
                            operation: [
                                'get',
                            ],
                            resource: [
                                'file',
                            ],
                        },
                    },
                    default: 0,
                    required: true,
                    description: 'ID of the file to get.',
                },
                {
                    displayName: 'Content',
                    name: 'content',
                    typeOptions: {
                        rows: 5,
                    },
                    type: 'string',
                    default: '',
                    required: true,
                    displayOptions: {
                        show: {
                            operation: [
                                'create',
                            ],
                            resource: [
                                'note',
                            ],
                        },
                    },
                    description: 'The content of the note to create',
                },
                {
                    displayName: 'Additional Fields',
                    name: 'additionalFields',
                    type: 'collection',
                    placeholder: 'Add Field',
                    displayOptions: {
                        show: {
                            operation: [
                                'create',
                                'getAll',
                            ],
                            resource: [
                                'note',
                            ],
                        },
                    },
                    default: {},
                    options: [
                        {
                            displayName: 'Deal ID',
                            name: 'deal_id',
                            type: 'number',
                            default: 0,
                            description: 'ID of the deal this note will be associated with',
                        },
                        {
                            displayName: 'Organization ID',
                            name: 'org_id',
                            type: 'number',
                            default: 0,
                            description: 'ID of the organization this note will be associated with.',
                        },
                        {
                            displayName: 'Person ID',
                            name: 'person_id',
                            type: 'number',
                            default: 0,
                            description: 'ID of the person this note will be associated with.',
                        },
                    ],
                },
                {
                    displayName: 'Note ID',
                    name: 'noteId',
                    type: 'number',
                    displayOptions: {
                        show: {
                            operation: [
                                'delete',
                            ],
                            resource: [
                                'note',
                            ],
                        },
                    },
                    default: 0,
                    required: true,
                    description: 'ID of the note to delete.',
                },
                {
                    displayName: 'Note ID',
                    name: 'noteId',
                    type: 'number',
                    displayOptions: {
                        show: {
                            operation: [
                                'get',
                            ],
                            resource: [
                                'note',
                            ],
                        },
                    },
                    default: 0,
                    required: true,
                    description: 'ID of the note to get.',
                },
                {
                    displayName: 'Note ID',
                    name: 'noteId',
                    type: 'number',
                    displayOptions: {
                        show: {
                            operation: [
                                'update',
                            ],
                            resource: [
                                'note',
                            ],
                        },
                    },
                    default: 0,
                    required: true,
                    description: 'ID of the note to update.',
                },
                {
                    displayName: 'Update Fields',
                    name: 'updateFields',
                    type: 'collection',
                    placeholder: 'Add Field',
                    displayOptions: {
                        show: {
                            operation: [
                                'update',
                            ],
                            resource: [
                                'note',
                            ],
                        },
                    },
                    default: {},
                    options: [
                        {
                            displayName: 'Content',
                            name: 'content',
                            typeOptions: {
                                rows: 5,
                            },
                            type: 'string',
                            default: '',
                            description: 'The content of the note',
                        },
                        {
                            displayName: 'Deal ID',
                            name: 'deal_id',
                            type: 'number',
                            default: 0,
                            description: 'ID of the deal this note will be associated with',
                        },
                        {
                            displayName: 'Organization ID',
                            name: 'org_id',
                            type: 'number',
                            default: 0,
                            description: 'ID of the organization this note will be associated with.',
                        },
                        {
                            displayName: 'Person ID',
                            name: 'person_id',
                            type: 'number',
                            default: 0,
                            description: 'ID of the person this note will be associated with.',
                        },
                    ],
                },
                {
                    displayName: 'Name',
                    name: 'name',
                    type: 'string',
                    default: '',
                    required: true,
                    displayOptions: {
                        show: {
                            operation: [
                                'create',
                            ],
                            resource: [
                                'organization',
                            ],
                        },
                    },
                    description: 'The name of the organization to create',
                },
                {
                    displayName: 'Additional Fields',
                    name: 'additionalFields',
                    type: 'collection',
                    placeholder: 'Add Field',
                    displayOptions: {
                        show: {
                            operation: [
                                'create',
                            ],
                            resource: [
                                'organization',
                            ],
                        },
                    },
                    default: {},
                    options: [
                        {
                            displayName: 'Visible to',
                            name: 'visible_to',
                            type: 'options',
                            options: [
                                {
                                    name: 'Owner & followers (private)',
                                    value: '1',
                                },
                                {
                                    name: 'Entire company (shared)',
                                    value: '3',
                                },
                            ],
                            default: '3',
                            description: 'Visibility of the person. If omitted, visibility will be set to the default visibility setting of this item type for the authorized user.',
                        },
                        {
                            displayName: 'Custom Properties',
                            name: 'customProperties',
                            placeholder: 'Add Custom Property',
                            description: 'Adds a custom property to set also values which have not been predefined.',
                            type: 'fixedCollection',
                            typeOptions: {
                                multipleValues: true,
                            },
                            default: {},
                            options: [
                                {
                                    name: 'property',
                                    displayName: 'Property',
                                    values: [
                                        {
                                            displayName: 'Property Name',
                                            name: 'name',
                                            type: 'string',
                                            default: '',
                                            description: 'Name of the property to set.',
                                        },
                                        {
                                            displayName: 'Property Value',
                                            name: 'value',
                                            type: 'string',
                                            default: '',
                                            description: 'Value of the property to set.',
                                        },
                                    ],
                                },
                            ],
                        },
                    ],
                },
                {
                    displayName: 'Organization ID',
                    name: 'organizationId',
                    type: 'number',
                    displayOptions: {
                        show: {
                            operation: [
                                'delete',
                            ],
                            resource: [
                                'organization',
                            ],
                        },
                    },
                    default: 0,
                    required: true,
                    description: 'ID of the organization to delete.',
                },
                {
                    displayName: 'Organization ID',
                    name: 'organizationId',
                    type: 'number',
                    displayOptions: {
                        show: {
                            operation: [
                                'get',
                            ],
                            resource: [
                                'organization',
                            ],
                        },
                    },
                    default: 0,
                    required: true,
                    description: 'ID of the organization to get.',
                },
                {
                    displayName: 'Name',
                    name: 'name',
                    type: 'string',
                    default: '',
                    required: true,
                    displayOptions: {
                        show: {
                            operation: [
                                'create',
                            ],
                            resource: [
                                'person',
                            ],
                        },
                    },
                    description: 'The name of the person to create',
                },
                {
                    displayName: 'Additional Fields',
                    name: 'additionalFields',
                    type: 'collection',
                    placeholder: 'Add Field',
                    displayOptions: {
                        show: {
                            operation: [
                                'create',
                            ],
                            resource: [
                                'person',
                            ],
                        },
                    },
                    default: {},
                    options: [
                        {
                            displayName: 'Email',
                            name: 'email',
                            type: 'string',
                            typeOptions: {
                                multipleValues: true,
                            },
                            default: '',
                            description: 'Email of the person.',
                        },
                        {
                            displayName: 'Organization ID',
                            name: 'org_id',
                            type: 'number',
                            default: 0,
                            description: 'ID of the organization this person will belong to.',
                        },
                        {
                            displayName: 'Phone',
                            name: 'phone',
                            type: 'string',
                            typeOptions: {
                                multipleValues: true,
                            },
                            default: '',
                            description: 'Phone number of the person.',
                        },
                        {
                            displayName: 'Visible to',
                            name: 'visible_to',
                            type: 'options',
                            options: [
                                {
                                    name: 'Owner & followers (private)',
                                    value: '1',
                                },
                                {
                                    name: 'Entire company (shared)',
                                    value: '3',
                                },
                            ],
                            default: '3',
                            description: 'Visibility of the person. If omitted, visibility will be set to the default visibility setting of this item type for the authorized user.',
                        },
                        {
                            displayName: 'Custom Properties',
                            name: 'customProperties',
                            placeholder: 'Add Custom Property',
                            description: 'Adds a custom property to set also values which have not been predefined.',
                            type: 'fixedCollection',
                            typeOptions: {
                                multipleValues: true,
                            },
                            default: {},
                            options: [
                                {
                                    name: 'property',
                                    displayName: 'Property',
                                    values: [
                                        {
                                            displayName: 'Property Name',
                                            name: 'name',
                                            type: 'string',
                                            default: '',
                                            description: 'Name of the property to set.',
                                        },
                                        {
                                            displayName: 'Property Value',
                                            name: 'value',
                                            type: 'string',
                                            default: '',
                                            description: 'Value of the property to set.',
                                        },
                                    ],
                                },
                            ],
                        },
                    ],
                },
                {
                    displayName: 'Person ID',
                    name: 'personId',
                    type: 'number',
                    displayOptions: {
                        show: {
                            operation: [
                                'delete',
                            ],
                            resource: [
                                'person',
                            ],
                        },
                    },
                    default: 0,
                    required: true,
                    description: 'ID of the person to delete.',
                },
                {
                    displayName: 'Person ID',
                    name: 'personId',
                    type: 'number',
                    displayOptions: {
                        show: {
                            operation: [
                                'get',
                            ],
                            resource: [
                                'person',
                            ],
                        },
                    },
                    default: 0,
                    required: true,
                    description: 'ID of the person to get.',
                },
                {
                    displayName: 'Person ID',
                    name: 'personId',
                    type: 'number',
                    displayOptions: {
                        show: {
                            operation: [
                                'update',
                            ],
                            resource: [
                                'person',
                            ],
                        },
                    },
                    default: 0,
                    required: true,
                    description: 'ID of the person to update.',
                },
                {
                    displayName: 'Update Fields',
                    name: 'updateFields',
                    type: 'collection',
                    description: 'The fields to update.',
                    placeholder: 'Add Field',
                    displayOptions: {
                        show: {
                            operation: [
                                'update',
                            ],
                            resource: [
                                'person',
                            ],
                        },
                    },
                    default: {},
                    options: [
                        {
                            displayName: 'Email',
                            name: 'email',
                            type: 'string',
                            typeOptions: {
                                multipleValues: true,
                            },
                            default: '',
                            description: 'Email of the person.',
                        },
                        {
                            displayName: 'Name',
                            name: 'name',
                            type: 'string',
                            default: '',
                            description: 'The name of the person',
                        },
                        {
                            displayName: 'Organization ID',
                            name: 'org_id',
                            type: 'number',
                            default: 0,
                            description: 'ID of the organization this person will belong to.',
                        },
                        {
                            displayName: 'Phone',
                            name: 'phone',
                            type: 'string',
                            typeOptions: {
                                multipleValues: true,
                            },
                            default: '',
                            description: 'Phone number of the person.',
                        },
                        {
                            displayName: 'Visible to',
                            name: 'visible_to',
                            type: 'options',
                            options: [
                                {
                                    name: 'Owner & followers (private)',
                                    value: '1',
                                },
                                {
                                    name: 'Entire company (shared)',
                                    value: '3',
                                },
                            ],
                            default: '3',
                            description: 'Visibility of the deal. If omitted, visibility will be set to the default visibility setting of this item type for the authorized user.',
                        },
                        {
                            displayName: 'Custom Properties',
                            name: 'customProperties',
                            placeholder: 'Add Custom Property',
                            description: 'Adds a custom property to set also values which have not been predefined.',
                            type: 'fixedCollection',
                            typeOptions: {
                                multipleValues: true,
                            },
                            default: {},
                            options: [
                                {
                                    name: 'property',
                                    displayName: 'Property',
                                    values: [
                                        {
                                            displayName: 'Property Name',
                                            name: 'name',
                                            type: 'string',
                                            default: '',
                                            description: 'Name of the property to set.',
                                        },
                                        {
                                            displayName: 'Property Value',
                                            name: 'value',
                                            type: 'string',
                                            default: '',
                                            description: 'Value of the property to set.',
                                        },
                                    ],
                                },
                            ],
                        },
                    ],
                },
                {
                    displayName: 'Resolve Properties',
                    name: 'resolveProperties',
                    type: 'boolean',
                    displayOptions: {
                        show: {
                            resource: [
                                'activity',
                                'deal',
                                'organization',
                                'person',
                                'product',
                            ],
                            operation: [
                                'get',
                                'getAll',
                            ],
                        },
                    },
                    default: false,
                    description: 'By default do custom properties get returned only as ID instead of their actual name. Also option fields contain only the ID instead of their actual value. If this option gets set they get automatically resolved.',
                },
                {
                    displayName: 'Encode Properties',
                    name: 'encodeProperties',
                    type: 'boolean',
                    displayOptions: {
                        show: {
                            resource: [
                                'activity',
                                'deal',
                                'organization',
                                'person',
                                'product',
                            ],
                            operation: [
                                'update',
                            ],
                        },
                    },
                    default: false,
                    description: 'By default do custom properties have to be set as ID instead of their actual name. Also option fields have to be set as ID instead of their actual value. If this option gets set they get automatically encoded.',
                },
                {
                    displayName: 'Return All',
                    name: 'returnAll',
                    type: 'boolean',
                    displayOptions: {
                        show: {
                            operation: [
                                'getAll',
                                'search',
                            ],
                        },
                    },
                    default: false,
                    description: 'If all results should be returned or only up to a given limit.',
                },
                {
                    displayName: 'Limit',
                    name: 'limit',
                    type: 'number',
                    displayOptions: {
                        show: {
                            operation: [
                                'getAll',
                                'search',
                            ],
                            returnAll: [
                                false,
                            ],
                        },
                    },
                    typeOptions: {
                        minValue: 1,
                        maxValue: 500,
                    },
                    default: 100,
                    description: 'How many results to return.',
                },
                {
                    displayName: 'Additional Fields',
                    name: 'additionalFields',
                    type: 'collection',
                    placeholder: 'Add Field',
                    displayOptions: {
                        show: {
                            operation: [
                                'getAll',
                            ],
                            resource: [
                                'person',
                            ],
                        },
                    },
                    default: {},
                    options: [
                        {
                            displayName: 'Filter ID',
                            name: 'filterId',
                            type: 'options',
                            typeOptions: {
                                loadOptionsMethod: 'getFilters',
                            },
                            default: '',
                            description: 'ID of the filter to use.',
                        },
                        {
                            displayName: 'First Char',
                            name: 'firstChar',
                            type: 'string',
                            default: '',
                            description: 'If supplied, only persons whose name starts with the specified letter will be returned ',
                        },
                    ],
                },
                {
                    displayName: 'Term',
                    name: 'term',
                    type: 'string',
                    required: true,
                    displayOptions: {
                        show: {
                            operation: [
                                'search',
                            ],
                            resource: [
                                'person',
                            ],
                        },
                    },
                    default: '',
                    description: 'The search term to look for. Minimum 2 characters (or 1 if using exact_match).',
                },
                {
                    displayName: 'Additional Fields',
                    name: 'additionalFields',
                    type: 'collection',
                    placeholder: 'Add Field',
                    displayOptions: {
                        show: {
                            operation: [
                                'search',
                            ],
                            resource: [
                                'person',
                            ],
                        },
                    },
                    default: {},
                    options: [
                        {
                            displayName: 'Exact Match',
                            name: 'exactMatch',
                            type: 'boolean',
                            default: false,
                            description: 'When enabled, only full exact matches against the given term are returned. It is not case sensitive.',
                        },
                        {
                            displayName: 'Fields',
                            name: 'fields',
                            type: 'string',
                            default: '',
                            description: 'A comma-separated string array. The fields to perform the search from. Defaults to all of them.',
                        },
                        {
                            displayName: 'Include Fields',
                            name: 'includeFields',
                            type: 'string',
                            default: '',
                            description: 'Supports including optional fields in the results which are not provided by default.',
                        },
                        {
                            displayName: 'Organization ID',
                            name: 'organizationId',
                            type: 'string',
                            default: '',
                            description: 'Will filter Deals by the provided Organization ID.',
                        },
                        {
                            displayName: 'RAW Data',
                            name: 'rawData',
                            type: 'boolean',
                            default: false,
                            description: `Returns the data exactly in the way it got received from the API.`,
                        },
                    ],
                },
                {
                    displayName: 'Additional Fields',
                    name: 'additionalFields',
                    type: 'collection',
                    placeholder: 'Add Field',
                    displayOptions: {
                        show: {
                            operation: [
                                'create',
                                'getAll',
                            ],
                            resource: [
                                'note',
                            ],
                        },
                    },
                    default: {},
                    options: [
                        {
                            displayName: 'Deal ID',
                            name: 'deal_id',
                            type: 'number',
                            default: 0,
                            description: 'ID of the deal this note will be associated with',
                        },
                        {
                            displayName: 'Organization ID',
                            name: 'org_id',
                            type: 'number',
                            default: 0,
                            description: 'ID of the organization this note will be associated with.',
                        },
                        {
                            displayName: 'Person ID',
                            name: 'person_id',
                            type: 'number',
                            default: 0,
                            description: 'ID of the person this note will be associated with.',
                        },
                    ],
                },
            ],
        };
        this.methods = {
            loadOptions: {
                async getFilters() {
                    const returnData = [];
                    const { data } = await GenericFunctions_1.pipedriveApiRequest.call(this, 'GET', '/filters', {}, { type: 'people' });
                    for (const filter of data) {
                        const filterName = filter.name;
                        const filterId = filter.id;
                        returnData.push({
                            name: filterName,
                            value: filterId,
                        });
                    }
                    return returnData;
                },
            },
        };
    }
    async execute() {
        const items = this.getInputData();
        const returnData = [];
        let body;
        let formData;
        let qs;
        let downloadFile;
        let requestMethod;
        let endpoint;
        let returnAll = false;
        const resource = this.getNodeParameter('resource', 0);
        const operation = this.getNodeParameter('operation', 0);
        let customProperties;
        if (['get', 'getAll', 'update'].includes(operation) && ['activity', 'deal', 'organization', 'person', 'product'].includes(resource)) {
            let getCustomProperties = false;
            if (['update'].includes(operation)) {
                getCustomProperties = this.getNodeParameter('encodeProperties', 0, false);
            }
            else {
                getCustomProperties = this.getNodeParameter('resolveProperties', 0, false);
            }
            if (getCustomProperties === true) {
                customProperties = await GenericFunctions_1.pipedriveGetCustomProperties.call(this, resource);
            }
        }
        for (let i = 0; i < items.length; i++) {
            requestMethod = 'GET';
            endpoint = '';
            downloadFile = false;
            body = {};
            formData = {};
            qs = {};
            if (resource === 'activity') {
                if (operation === 'create') {
                    requestMethod = 'POST';
                    endpoint = '/activities';
                    body.subject = this.getNodeParameter('subject', i);
                    body.done = this.getNodeParameter('done', i);
                    body.type = this.getNodeParameter('type', i);
                    const additionalFields = this.getNodeParameter('additionalFields', i);
                    addAdditionalFields(body, additionalFields);
                }
                else if (operation === 'delete') {
                    requestMethod = 'DELETE';
                    const activityId = this.getNodeParameter('activityId', i);
                    endpoint = `/activities/${activityId}`;
                }
                else if (operation === 'get') {
                    requestMethod = 'GET';
                    const activityId = this.getNodeParameter('activityId', i);
                    endpoint = `/activities/${activityId}`;
                }
                else if (operation === 'getAll') {
                    requestMethod = 'GET';
                    returnAll = this.getNodeParameter('returnAll', i);
                    if (returnAll === false) {
                        qs.limit = this.getNodeParameter('limit', i);
                    }
                    endpoint = `/activities`;
                }
                else if (operation === 'update') {
                    requestMethod = 'PUT';
                    const activityId = this.getNodeParameter('activityId', i);
                    endpoint = `/activities/${activityId}`;
                    const updateFields = this.getNodeParameter('updateFields', i);
                    addAdditionalFields(body, updateFields);
                }
            }
            else if (resource === 'deal') {
                if (operation === 'create') {
                    requestMethod = 'POST';
                    endpoint = '/deals';
                    body.title = this.getNodeParameter('title', i);
                    const additionalFields = this.getNodeParameter('additionalFields', i);
                    addAdditionalFields(body, additionalFields);
                }
                else if (operation === 'delete') {
                    requestMethod = 'DELETE';
                    const dealId = this.getNodeParameter('dealId', i);
                    endpoint = `/deals/${dealId}`;
                }
                else if (operation === 'duplicate') {
                    requestMethod = 'POST';
                    const dealId = this.getNodeParameter('dealId', i);
                    endpoint = `/deals/${dealId}/duplicate`;
                }
                else if (operation === 'get') {
                    requestMethod = 'GET';
                    const dealId = this.getNodeParameter('dealId', i);
                    endpoint = `/deals/${dealId}`;
                }
                else if (operation === 'getAll') {
                    requestMethod = 'GET';
                    returnAll = this.getNodeParameter('returnAll', i);
                    if (returnAll === false) {
                        qs.limit = this.getNodeParameter('limit', i);
                    }
                    endpoint = `/deals`;
                }
                else if (operation === 'update') {
                    requestMethod = 'PUT';
                    const dealId = this.getNodeParameter('dealId', i);
                    endpoint = `/deals/${dealId}`;
                    const updateFields = this.getNodeParameter('updateFields', i);
                    addAdditionalFields(body, updateFields);
                }
            }
            else if (resource === 'file') {
                if (operation === 'create') {
                    requestMethod = 'POST';
                    endpoint = '/files';
                    const item = items[i];
                    if (item.binary === undefined) {
                        throw new Error('No binary data exists on item!');
                    }
                    const binaryPropertyName = this.getNodeParameter('binaryPropertyName', i);
                    if (item.binary[binaryPropertyName] === undefined) {
                        throw new Error(`No binary data property "${binaryPropertyName}" does not exists on item!`);
                    }
                    const fileBufferData = Buffer.from(item.binary[binaryPropertyName].data, n8n_core_1.BINARY_ENCODING);
                    formData.file = {
                        value: fileBufferData,
                        options: {
                            contentType: item.binary[binaryPropertyName].mimeType,
                            filename: item.binary[binaryPropertyName].fileName,
                        },
                    };
                    const additionalFields = this.getNodeParameter('additionalFields', i);
                    addAdditionalFields(formData, additionalFields);
                }
                else if (operation === 'delete') {
                    requestMethod = 'DELETE';
                    const fileId = this.getNodeParameter('fileId', i);
                    endpoint = `/files/${fileId}`;
                }
                else if (operation === 'download') {
                    requestMethod = 'GET';
                    downloadFile = true;
                    const fileId = this.getNodeParameter('fileId', i);
                    endpoint = `/files/${fileId}/download`;
                }
                else if (operation === 'get') {
                    requestMethod = 'GET';
                    const fileId = this.getNodeParameter('fileId', i);
                    endpoint = `/files/${fileId}`;
                }
            }
            else if (resource === 'note') {
                if (operation === 'create') {
                    requestMethod = 'POST';
                    endpoint = '/notes';
                    body.content = this.getNodeParameter('content', i);
                    const additionalFields = this.getNodeParameter('additionalFields', i);
                    addAdditionalFields(body, additionalFields);
                }
                else if (operation === 'delete') {
                    requestMethod = 'DELETE';
                    const noteId = this.getNodeParameter('noteId', i);
                    endpoint = `/notes/${noteId}`;
                }
                else if (operation === 'get') {
                    requestMethod = 'GET';
                    const noteId = this.getNodeParameter('noteId', i);
                    endpoint = `/notes/${noteId}`;
                }
                else if (operation === 'getAll') {
                    requestMethod = 'GET';
                    endpoint = `/notes`;
                    returnAll = this.getNodeParameter('returnAll', i);
                    if (returnAll === false) {
                        qs.limit = this.getNodeParameter('limit', i);
                    }
                    const additionalFields = this.getNodeParameter('additionalFields', i);
                    addAdditionalFields(qs, additionalFields);
                }
                else if (operation === 'update') {
                    requestMethod = 'PUT';
                    const noteId = this.getNodeParameter('noteId', i);
                    endpoint = `/notes/${noteId}`;
                    const updateFields = this.getNodeParameter('updateFields', i);
                    addAdditionalFields(body, updateFields);
                }
            }
            else if (resource === 'organization') {
                if (operation === 'create') {
                    requestMethod = 'POST';
                    endpoint = '/organizations';
                    body.name = this.getNodeParameter('name', i);
                    const additionalFields = this.getNodeParameter('additionalFields', i);
                    addAdditionalFields(body, additionalFields);
                }
                else if (operation === 'delete') {
                    requestMethod = 'DELETE';
                    const organizationId = this.getNodeParameter('organizationId', i);
                    endpoint = `/organizations/${organizationId}`;
                }
                else if (operation === 'get') {
                    requestMethod = 'GET';
                    const organizationId = this.getNodeParameter('organizationId', i);
                    endpoint = `/organizations/${organizationId}`;
                }
                else if (operation === 'getAll') {
                    requestMethod = 'GET';
                    returnAll = this.getNodeParameter('returnAll', i);
                    if (returnAll === false) {
                        qs.limit = this.getNodeParameter('limit', i);
                    }
                    endpoint = `/organizations`;
                }
            }
            else if (resource === 'person') {
                if (operation === 'create') {
                    requestMethod = 'POST';
                    endpoint = '/persons';
                    body.name = this.getNodeParameter('name', i);
                    const additionalFields = this.getNodeParameter('additionalFields', i);
                    addAdditionalFields(body, additionalFields);
                }
                else if (operation === 'delete') {
                    requestMethod = 'DELETE';
                    const personId = this.getNodeParameter('personId', i);
                    endpoint = `/persons/${personId}`;
                }
                else if (operation === 'get') {
                    requestMethod = 'GET';
                    const personId = this.getNodeParameter('personId', i);
                    endpoint = `/persons/${personId}`;
                }
                else if (operation === 'getAll') {
                    requestMethod = 'GET';
                    returnAll = this.getNodeParameter('returnAll', i);
                    if (returnAll === false) {
                        qs.limit = this.getNodeParameter('limit', i);
                    }
                    const additionalFields = this.getNodeParameter('additionalFields', i);
                    if (additionalFields.filterId) {
                        qs.filter_id = additionalFields.filterId;
                    }
                    if (additionalFields.firstChar) {
                        qs.first_char = additionalFields.firstChar;
                    }
                    endpoint = `/persons`;
                }
                else if (operation === 'search') {
                    requestMethod = 'GET';
                    qs.term = this.getNodeParameter('term', i);
                    returnAll = this.getNodeParameter('returnAll', i);
                    if (returnAll === false) {
                        qs.limit = this.getNodeParameter('limit', i);
                    }
                    const additionalFields = this.getNodeParameter('additionalFields', i);
                    if (additionalFields.fields) {
                        qs.fields = additionalFields.fields;
                    }
                    if (additionalFields.exactMatch) {
                        qs.exact_match = additionalFields.exactMatch;
                    }
                    if (additionalFields.organizationId) {
                        qs.organization_id = parseInt(additionalFields.organizationId, 10);
                    }
                    if (additionalFields.includeFields) {
                        qs.include_fields = additionalFields.includeFields;
                    }
                    endpoint = `/persons/search`;
                }
                else if (operation === 'update') {
                    requestMethod = 'PUT';
                    const personId = this.getNodeParameter('personId', i);
                    endpoint = `/persons/${personId}`;
                    const updateFields = this.getNodeParameter('updateFields', i);
                    addAdditionalFields(body, updateFields);
                }
            }
            else if (resource === 'product') {
                if (operation === 'getAll') {
                    requestMethod = 'GET';
                    returnAll = this.getNodeParameter('returnAll', i);
                    if (returnAll === false) {
                        qs.limit = this.getNodeParameter('limit', i);
                    }
                    endpoint = `/products`;
                }
            }
            else {
                throw new Error(`The resource "${resource}" is not known!`);
            }
            let responseData;
            if (returnAll === true) {
                responseData = await GenericFunctions_1.pipedriveApiRequestAllItems.call(this, requestMethod, endpoint, body, qs);
            }
            else {
                if (customProperties !== undefined) {
                    GenericFunctions_1.pipedriveEncodeCustomProperties(customProperties, body);
                }
                responseData = await GenericFunctions_1.pipedriveApiRequest.call(this, requestMethod, endpoint, body, qs, formData, downloadFile);
            }
            if (resource === 'file' && operation === 'download') {
                const newItem = {
                    json: items[i].json,
                    binary: {},
                };
                if (items[i].binary !== undefined) {
                    Object.assign(newItem.binary, items[i].binary);
                }
                items[i] = newItem;
                const binaryPropertyName = this.getNodeParameter('binaryPropertyName', i);
                items[i].binary[binaryPropertyName] = await this.helpers.prepareBinaryData(responseData.data);
            }
            else {
                if (responseData.data === null) {
                    responseData.data = [];
                }
                if (operation === 'search' && responseData.data && responseData.data.items) {
                    responseData.data = responseData.data.items;
                    const additionalFields = this.getNodeParameter('additionalFields', i);
                    if (additionalFields.rawData !== true) {
                        responseData.data = responseData.data.map((item) => {
                            return Object.assign({ result_score: item.result_score }, item.item);
                        });
                    }
                }
                if (Array.isArray(responseData.data)) {
                    returnData.push.apply(returnData, responseData.data);
                }
                else {
                    returnData.push(responseData.data);
                }
            }
        }
        if (customProperties !== undefined) {
            for (const item of returnData) {
                await GenericFunctions_1.pipedriveResolveCustomProperties(customProperties, item);
            }
        }
        if (resource === 'file' && operation === 'download') {
            return this.prepareOutputData(items);
        }
        else {
            return [this.helpers.returnJsonArray(returnData)];
        }
    }
}
exports.Pipedrive = Pipedrive;
//# sourceMappingURL=Pipedrive.node.js.map