"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Pushover = void 0;
const n8n_core_1 = require("n8n-core");
const GenericFunctions_1 = require("./GenericFunctions");
class Pushover {
    constructor() {
        this.description = {
            displayName: 'Pushover',
            name: 'pushover',
            icon: 'file:pushover.png',
            group: ['input'],
            version: 1,
            subtitle: '={{$parameter["operation"] + ": " + $parameter["resource"]}}',
            description: 'Consume Pushover API.',
            defaults: {
                name: 'Pushover',
                color: '#4b9cea',
            },
            inputs: ['main'],
            outputs: ['main'],
            credentials: [
                {
                    name: 'pushoverApi',
                    required: true,
                },
            ],
            properties: [
                {
                    displayName: 'Resource',
                    name: 'resource',
                    type: 'options',
                    options: [
                        {
                            name: 'Message',
                            value: 'message',
                        },
                    ],
                    default: 'message',
                    description: 'The resource to operate on.',
                },
                {
                    displayName: 'Operation',
                    name: 'operation',
                    type: 'options',
                    displayOptions: {
                        show: {
                            resource: [
                                'message',
                            ],
                        },
                    },
                    options: [
                        {
                            name: 'Push',
                            value: 'push',
                        },
                    ],
                    default: 'push',
                    description: 'The resource to operate on.',
                },
                {
                    displayName: 'User Key',
                    name: 'userKey',
                    type: 'string',
                    required: true,
                    displayOptions: {
                        show: {
                            resource: [
                                'message',
                            ],
                            operation: [
                                'push',
                            ],
                        },
                    },
                    default: '',
                    description: `The user/group key (not e-mail address) of your user (or you),<br>
				viewable when logged into our <a href="https://pushover.net/" target="_blank">dashboard</a> (often referred to as USER_KEY in our <a href="https://support.pushover.net/i44-example-code-and-pushover-libraries" target="_blank"></a> and code examples)`,
                },
                {
                    displayName: 'Message',
                    name: 'message',
                    type: 'string',
                    required: true,
                    displayOptions: {
                        show: {
                            resource: [
                                'message',
                            ],
                            operation: [
                                'push',
                            ],
                        },
                    },
                    default: '',
                    description: `Your message`,
                },
                {
                    displayName: 'Priority',
                    name: 'priority',
                    type: 'options',
                    displayOptions: {
                        show: {
                            resource: [
                                'message',
                            ],
                            operation: [
                                'push',
                            ],
                        },
                    },
                    options: [
                        {
                            name: 'Lowest Priority',
                            value: -2,
                        },
                        {
                            name: 'Low Priority',
                            value: -1,
                        },
                        {
                            name: 'Normal Priority',
                            value: 0,
                        },
                        {
                            name: 'High Priority',
                            value: 1,
                        },
                        {
                            name: 'Emergency Priority',
                            value: 2,
                        },
                    ],
                    default: -2,
                    description: `send as -2 to generate no notification/alert,<br>
				-1 to always send as a quiet notification,<br>
				1 to display as high-priority and bypass the user's quiet hours, or<br>
				2 to also require confirmation from the user`,
                },
                {
                    displayName: 'Retry (seconds)',
                    name: 'retry',
                    type: 'number',
                    typeOptions: {
                        minValue: 0,
                    },
                    required: true,
                    displayOptions: {
                        show: {
                            resource: [
                                'message',
                            ],
                            operation: [
                                'push',
                            ],
                            priority: [
                                2,
                            ],
                        },
                    },
                    default: 30,
                    description: `Specifies how often (in seconds) the Pushover servers will send the same notification to the user.<br>
				This parameter must have a value of at least 30 seconds between retries.`,
                },
                {
                    displayName: 'Expire (seconds)',
                    name: 'expire',
                    type: 'number',
                    typeOptions: {
                        minValue: 0,
                        maxValue: 10800,
                    },
                    required: true,
                    displayOptions: {
                        show: {
                            resource: [
                                'message',
                            ],
                            operation: [
                                'push',
                            ],
                            priority: [
                                2,
                            ],
                        },
                    },
                    default: 30,
                    description: `Specifies how many seconds your notification will continue to be retried for (every retry seconds)`,
                },
                {
                    displayName: 'Additional Fields',
                    name: 'additionalFields',
                    type: 'collection',
                    placeholder: 'Add Field',
                    displayOptions: {
                        show: {
                            resource: [
                                'message',
                            ],
                            operation: [
                                'push',
                            ],
                        },
                    },
                    default: {},
                    options: [
                        {
                            displayName: 'Attachment',
                            name: 'attachmentsUi',
                            placeholder: 'Add Attachments',
                            type: 'fixedCollection',
                            typeOptions: {
                                multipleValues: false,
                            },
                            options: [
                                {
                                    name: 'attachmentsValues',
                                    displayName: 'Attachment Property',
                                    values: [
                                        {
                                            displayName: 'Binary Property',
                                            name: 'binaryPropertyName',
                                            type: 'string',
                                            default: '',
                                            placeholder: 'data',
                                            description: 'Name of the binary properties which contain data which should be added to email as attachment',
                                        },
                                    ],
                                },
                            ],
                            default: '',
                        },
                        {
                            displayName: 'Device',
                            name: 'device',
                            type: 'string',
                            default: '',
                            description: `Your user's device name to send the message directly to that device,<br>
						rather than all of the user's devices (multiple devices may be separated by a comma)`,
                        },
                        {
                            displayName: 'Sound',
                            name: 'sound',
                            type: 'options',
                            typeOptions: {
                                loadOptionsMethod: 'getSounds',
                            },
                            default: '',
                            description: `The name of one of the sounds supported by device clients to override the user's default sound choice`,
                        },
                        {
                            displayName: 'Title',
                            name: 'title',
                            type: 'string',
                            default: '',
                            description: `Your message's title, otherwise your app's name is used`,
                        },
                        {
                            displayName: 'Timestamp',
                            name: 'timestamp',
                            type: 'dateTime',
                            default: '',
                            description: `A Unix timestamp of your message's date and time to display to the user, rather than the time your message is received by our API`,
                        },
                        {
                            displayName: 'URL',
                            name: 'url',
                            type: 'string',
                            default: '',
                            description: `a supplementary URL to show with your message`,
                        },
                        {
                            displayName: 'URL Title',
                            name: 'url_title',
                            type: 'string',
                            default: '',
                            description: `A title for your supplementary URL, otherwise just the URL is shown`,
                        },
                    ],
                },
            ],
        };
        this.methods = {
            loadOptions: {
                async getSounds() {
                    const { sounds } = await GenericFunctions_1.pushoverApiRequest.call(this, 'GET', '/sounds.json', {});
                    const returnData = [];
                    for (const key of Object.keys(sounds)) {
                        returnData.push({
                            name: sounds[key],
                            value: key,
                        });
                    }
                    return returnData;
                },
            },
        };
    }
    async execute() {
        const items = this.getInputData();
        const returnData = [];
        const length = items.length;
        const qs = {};
        let responseData;
        const resource = this.getNodeParameter('resource', 0);
        const operation = this.getNodeParameter('operation', 0);
        for (let i = 0; i < length; i++) {
            if (resource === 'message') {
                if (operation === 'push') {
                    const userKey = this.getNodeParameter('userKey', i);
                    const message = this.getNodeParameter('message', i);
                    const priority = this.getNodeParameter('priority', i);
                    const additionalFields = this.getNodeParameter('additionalFields', i);
                    const body = {
                        user: userKey,
                        message,
                        priority,
                    };
                    if (priority === 2) {
                        body.retry = this.getNodeParameter('retry', i);
                        body.expire = this.getNodeParameter('expire', i);
                    }
                    Object.assign(body, additionalFields);
                    if (body.attachmentsUi) {
                        const attachment = body.attachmentsUi.attachmentsValues;
                        if (attachment) {
                            const binaryPropertyName = attachment.binaryPropertyName;
                            if (items[i].binary === undefined) {
                                throw new Error('No binary data exists on item!');
                            }
                            const item = items[i].binary;
                            const binaryData = item[binaryPropertyName];
                            if (binaryData === undefined) {
                                throw new Error(`No binary data property "${binaryPropertyName}" does not exists on item!`);
                            }
                            body.attachment = {
                                value: Buffer.from(binaryData.data, n8n_core_1.BINARY_ENCODING),
                                options: {
                                    filename: binaryData.fileName,
                                },
                            };
                            delete body.attachmentsUi;
                        }
                    }
                    responseData = await GenericFunctions_1.pushoverApiRequest.call(this, 'POST', `/messages.json`, body);
                }
            }
        }
        if (Array.isArray(responseData)) {
            returnData.push.apply(returnData, responseData);
        }
        else if (responseData !== undefined) {
            returnData.push(responseData);
        }
        return [this.helpers.returnJsonArray(returnData)];
    }
}
exports.Pushover = Pushover;
//# sourceMappingURL=Pushover.node.js.map